@@ -26,8 +26,11 @@ echo "Installing Stationary"
 mkdir -p "/Developer/ProjectBuilder Extras/Project Templates/Application"
 mkdir -p "/Developer/ProjectBuilder Extras/Target Templates/SDL"
 
-cp -r "$ROOT/Project Stationary/SDL Application" "/Developer/ProjectBuilder Extras/Project Templates/Application/"
-cp "$ROOT/Project Stationary/Application.trgttmpl" "/Developer/ProjectBuilder Extras/Target Templates/SDL/"
+cp -r "$ROOT/Project Stationary/SDL Application"              "/Developer/ProjectBuilder Extras/Project Templates/Application/"
+cp -r "$ROOT/Project Stationary/SDL Cocoa Application"        "/Developer/ProjectBuilder Extras/Project Templates/Application/"
+cp -r "$ROOT/Project Stationary/SDL Custom Cocoa Application" "/Developer/ProjectBuilder Extras/Project Templates/Application/"
+cp -r "$ROOT/Project Stationary/SDL OpenGL Application"       "/Developer/ProjectBuilder Extras/Project Templates/Application/"
+cp "$ROOT/Project Stationary/Application.trgttmpl"            "/Developer/ProjectBuilder Extras/Target Templates/SDL/"
 
 rm -rf "$ROOT/Project Stationary"
 
