@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_mutex.h,v 1.5.2.8 2001/02/17 01:45:30 hercules Exp $";
+ "@(#) $Id: SDL_mutex.h,v 1.6 2002/06/13 23:59:20 slouken Exp $";
 #endif
 
 #ifndef _SDL_mutex_h
@@ -60,18 +60,21 @@ struct SDL_mutex;
 typedef struct SDL_mutex SDL_mutex;
 
 /* Create a mutex, initialized unlocked */
-extern DECLSPEC SDL_mutex * SDL_CreateMutex(void);
+extern DECLSPEC SDL_mutex * SDLCALL SDL_CreateMutex(void);
 
 /* Lock the mutex  (Returns 0, or -1 on error) */
 #define SDL_LockMutex(m)	SDL_mutexP(m)
-extern DECLSPEC int SDL_mutexP(SDL_mutex *mutex);
+extern DECLSPEC int SDLCALL SDL_mutexP(SDL_mutex *mutex);
 
-/* Unlock the mutex  (Returns 0, or -1 on error) */
+/* Unlock the mutex  (Returns 0, or -1 on error)
+   It is an error to unlock a mutex that has not been locked by
+   the current thread, and doing so results in undefined behavior.
+ */
 #define SDL_UnlockMutex(m)	SDL_mutexV(m)
-extern DECLSPEC int SDL_mutexV(SDL_mutex *mutex);
+extern DECLSPEC int SDLCALL SDL_mutexV(SDL_mutex *mutex);
 
 /* Destroy a mutex */
-extern DECLSPEC void SDL_DestroyMutex(SDL_mutex *mutex);
+extern DECLSPEC void SDLCALL SDL_DestroyMutex(SDL_mutex *mutex);
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -83,21 +86,21 @@ struct SDL_semaphore;
 typedef struct SDL_semaphore SDL_sem;
 
 /* Create a semaphore, initialized with value, returns NULL on failure. */
-extern DECLSPEC SDL_sem * SDL_CreateSemaphore(Uint32 initial_value);
+extern DECLSPEC SDL_sem * SDLCALL SDL_CreateSemaphore(Uint32 initial_value);
 
 /* Destroy a semaphore */
-extern DECLSPEC void SDL_DestroySemaphore(SDL_sem *sem);
+extern DECLSPEC void SDLCALL SDL_DestroySemaphore(SDL_sem *sem);
 
 /* This function suspends the calling thread until the semaphore pointed 
  * to by sem has a positive count. It then atomically decreases the semaphore
  * count.
  */
-extern DECLSPEC int SDL_SemWait(SDL_sem *sem);
+extern DECLSPEC int SDLCALL SDL_SemWait(SDL_sem *sem);
 
 /* Non-blocking variant of SDL_SemWait(), returns 0 if the wait succeeds,
    SDL_MUTEX_TIMEDOUT if the wait would block, and -1 on error.
 */
-extern DECLSPEC int SDL_SemTryWait(SDL_sem *sem);
+extern DECLSPEC int SDLCALL SDL_SemTryWait(SDL_sem *sem);
 
 /* Variant of SDL_SemWait() with a timeout in milliseconds, returns 0 if
    the wait succeeds, SDL_MUTEX_TIMEDOUT if the wait does not succeed in
@@ -105,15 +108,15 @@ extern DECLSPEC int SDL_SemTryWait(SDL_sem *sem);
    On some platforms this function is implemented by looping with a delay
    of 1 ms, and so should be avoided if possible.
 */
-extern DECLSPEC int SDL_SemWaitTimeout(SDL_sem *sem, Uint32 ms);
+extern DECLSPEC int SDLCALL SDL_SemWaitTimeout(SDL_sem *sem, Uint32 ms);
 
 /* Atomically increases the semaphore's count (not blocking), returns 0,
    or -1 on error.
  */
-extern DECLSPEC int SDL_SemPost(SDL_sem *sem);
+extern DECLSPEC int SDLCALL SDL_SemPost(SDL_sem *sem);
 
 /* Returns the current count of the semaphore */
-extern DECLSPEC Uint32 SDL_SemValue(SDL_sem *sem);
+extern DECLSPEC Uint32 SDLCALL SDL_SemValue(SDL_sem *sem);
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -125,26 +128,26 @@ struct SDL_cond;
 typedef struct SDL_cond SDL_cond;
 
 /* Create a condition variable */
-extern DECLSPEC SDL_cond * SDL_CreateCond(void);
+extern DECLSPEC SDL_cond * SDLCALL SDL_CreateCond(void);
 
 /* Destroy a condition variable */
-extern DECLSPEC void SDL_DestroyCond(SDL_cond *cond);
+extern DECLSPEC void SDLCALL SDL_DestroyCond(SDL_cond *cond);
 
 /* Restart one of the threads that are waiting on the condition variable,
    returns 0 or -1 on error.
  */
-extern DECLSPEC int SDL_CondSignal(SDL_cond *cond);
+extern DECLSPEC int SDLCALL SDL_CondSignal(SDL_cond *cond);
 
 /* Restart all threads that are waiting on the condition variable,
    returns 0 or -1 on error.
  */
-extern DECLSPEC int SDL_CondBroadcast(SDL_cond *cond);
+extern DECLSPEC int SDLCALL SDL_CondBroadcast(SDL_cond *cond);
 
 /* Wait on the condition variable, unlocking the provided mutex.
    The mutex must be locked before entering this function!
    Returns 0 when it is signaled, or -1 on error.
  */
-extern DECLSPEC int SDL_CondWait(SDL_cond *cond, SDL_mutex *mut);
+extern DECLSPEC int SDLCALL SDL_CondWait(SDL_cond *cond, SDL_mutex *mut);
 
 /* Waits for at most 'ms' milliseconds, and returns 0 if the condition
    variable is signaled, SDL_MUTEX_TIMEDOUT if the condition is not
@@ -152,7 +155,7 @@ extern DECLSPEC int SDL_CondWait(SDL_cond *cond, SDL_mutex *mut);
    On some platforms this function is implemented by looping with a delay
    of 1 ms, and so should be avoided if possible.
 */
-extern DECLSPEC int SDL_CondWaitTimeout(SDL_cond *cond, SDL_mutex *mutex, Uint32 ms);
+extern DECLSPEC int SDLCALL SDL_CondWaitTimeout(SDL_cond *cond, SDL_mutex *mutex, Uint32 ms);
 
 /* Ends C function definitions when using C++ */
 #ifdef __cplusplus
