@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_video.h,v 1.5.2.32 2000/09/24 00:59:46 hercules Exp $";
+ "@(#) $Id: SDL_video.h,v 1.16 2003/08/06 20:12:00 slouken Exp $";
 #endif
 
 /* Header file for access to the SDL raw framebuffer window */
@@ -58,6 +58,7 @@ typedef struct {
 	Uint8 b;
 	Uint8 unused;
 } SDL_Color;
+#define SDL_Colour SDL_Color
 
 typedef struct {
 	int       ncolors;
@@ -110,13 +111,15 @@ typedef struct SDL_Surface {
 	/* clipping information */
 	SDL_Rect clip_rect;			/* Read-only */
 	Uint32 unused1;				/* for binary compatibility */
-	Uint32 unused2;				/* for binary compatibility */
+
+	/* Allow recursive locks */
+	Uint32 locked;				/* Private */
 
 	/* info for fast blit mapping to other surfaces */
 	struct SDL_BlitMap *map;		/* Private */
 
-        /* format version, bumped at every change to invalidate blit maps */
-        unsigned int format_version;		/* Private */
+	/* format version, bumped at every change to invalidate blit maps */
+	unsigned int format_version;		/* Private */
 
 	/* Reference count -- used when freeing surface */
 	int refcount;				/* Read-mostly */
@@ -135,11 +138,12 @@ typedef struct SDL_Surface {
 #define SDL_OPENGL      0x00000002      /* Create an OpenGL rendering context */
 #define SDL_OPENGLBLIT	0x0000000A	/* Create an OpenGL rendering context and use it for blitting */
 #define SDL_RESIZABLE	0x00000010	/* This video mode may be resized */
+#define SDL_NOFRAME	0x00000020	/* No window caption or edge frame */
 /* Used internally (read-only) */
 #define SDL_HWACCEL	0x00000100	/* Blit uses hardware acceleration */
 #define SDL_SRCCOLORKEY	0x00001000	/* Blit uses a source color key */
 #define SDL_RLEACCELOK	0x00002000	/* Private flag */
-#define SDL_RLEACCEL	0x00004000	/* Colorkey blit is RLE accelerated */
+#define SDL_RLEACCEL	0x00004000	/* Surface is RLE encoded */
 #define SDL_SRCALPHA	0x00010000	/* Blit uses source alpha blending */
 #define SDL_PREALLOC	0x01000000	/* Surface uses preallocated memory */
 
@@ -212,7 +216,10 @@ typedef enum {
     SDL_GL_ACCUM_RED_SIZE,
     SDL_GL_ACCUM_GREEN_SIZE,
     SDL_GL_ACCUM_BLUE_SIZE,
-    SDL_GL_ACCUM_ALPHA_SIZE
+    SDL_GL_ACCUM_ALPHA_SIZE,
+    SDL_GL_STEREO,
+    SDL_GL_MULTISAMPLEBUFFERS,
+    SDL_GL_MULTISAMPLESAMPLES
 } SDL_GLattr;
 
 /* flags for SDL_SetPalette() */
@@ -223,6 +230,7 @@ typedef enum {
 
 /* These functions are used internally, and should not be used unless you
  * have a specific need to specify the video driver you want to use.
+ * You should normally use SDL_Init() or SDL_InitSubSystem().
  *
  * SDL_VideoInit() initializes the video subsystem -- sets up a connection
  * to the window manager, etc, and determines the current video mode and
@@ -233,14 +241,14 @@ typedef enum {
  * SDL_Init() before opening the sound device, otherwise under Win32 DirectX,
  * you won't be able to set full-screen display modes.
  */
-extern DECLSPEC int SDL_VideoInit(const char *driver_name, Uint32 flags);
-extern DECLSPEC void SDL_VideoQuit(void);
+extern DECLSPEC int SDLCALL SDL_VideoInit(const char *driver_name, Uint32 flags);
+extern DECLSPEC void SDLCALL SDL_VideoQuit(void);
 
 /* This function fills the given character buffer with the name of the
  * video driver, and returns a pointer to it if the video driver has
  * been initialized.  It returns NULL if no driver has been initialized.
  */
-extern DECLSPEC char *SDL_VideoDriverName(char *namebuf, int maxlen);
+extern DECLSPEC char * SDLCALL SDL_VideoDriverName(char *namebuf, int maxlen);
 
 /*
  * This function returns a pointer to the current display surface.
@@ -248,7 +256,7 @@ extern DECLSPEC char *SDL_VideoDriverName(char *namebuf, int maxlen);
  * function returns the publicly visible surface, not the real video
  * surface.
  */
-extern DECLSPEC SDL_Surface * SDL_GetVideoSurface(void);
+extern DECLSPEC SDL_Surface * SDLCALL SDL_GetVideoSurface(void);
 
 /*
  * This function returns a read-only pointer to information about the
@@ -256,7 +264,7 @@ extern DECLSPEC SDL_Surface * SDL_GetVideoSurface(void);
  * member of the returned structure will contain the pixel format of the
  * "best" video mode.
  */
-extern DECLSPEC const SDL_VideoInfo * SDL_GetVideoInfo(void);
+extern DECLSPEC const SDL_VideoInfo * SDLCALL SDL_GetVideoInfo(void);
 
 /* 
  * Check to see if a particular video mode is supported.
@@ -269,7 +277,7 @@ extern DECLSPEC const SDL_VideoInfo * SDL_GetVideoInfo(void);
  * The arguments to SDL_VideoModeOK() are the same ones you would pass to
  * SDL_SetVideoMode()
  */
-extern DECLSPEC int SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags);
+extern DECLSPEC int SDLCALL SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags);
 
 /*
  * Return a pointer to an array of available screen dimensions for the
@@ -280,7 +288,7 @@ extern DECLSPEC int SDL_VideoModeOK(int width, int height, int bpp, Uint32 flags
  * If 'format' is NULL, the mode list will be for the format given 
  * by SDL_GetVideoInfo()->vfmt
  */
-extern DECLSPEC SDL_Rect ** SDL_ListModes(SDL_PixelFormat *format, Uint32 flags);
+extern DECLSPEC SDL_Rect ** SDLCALL SDL_ListModes(SDL_PixelFormat *format, Uint32 flags);
 
 /*
  * Set up a video mode with the specified width, height and bits-per-pixel.
@@ -323,9 +331,24 @@ extern DECLSPEC SDL_Rect ** SDL_ListModes(SDL_PixelFormat *format, Uint32 flags)
  * memory while the monitor is refreshing.  It should only be used by 
  * applications that redraw the entire screen on every update.
  *
+ * If SDL_RESIZABLE is set in 'flags', the SDL library will allow the
+ * window manager, if any, to resize the window at runtime.  When this
+ * occurs, SDL will send a SDL_VIDEORESIZE event to you application,
+ * and you must respond to the event by re-calling SDL_SetVideoMode()
+ * with the requested size (or another size that suits the application).
+ *
+ * If SDL_NOFRAME is set in 'flags', the SDL library will create a window
+ * without any title bar or frame decoration.  Fullscreen video modes have
+ * this flag set automatically.
+ *
  * This function returns the video framebuffer surface, or NULL if it fails.
+ *
+ * If you rely on functionality provided by certain video flags, check the
+ * flags of the returned surface to make sure that functionality is available.
+ * SDL will fall back to reduced functionality if the exact flags you wanted
+ * are not available.
  */
-extern DECLSPEC SDL_Surface *SDL_SetVideoMode
+extern DECLSPEC SDL_Surface * SDLCALL SDL_SetVideoMode
 			(int width, int height, int bpp, Uint32 flags);
 
 /*
@@ -334,9 +357,9 @@ extern DECLSPEC SDL_Surface *SDL_SetVideoMode
  * screen.
  * These functions should not be called while 'screen' is locked.
  */
-extern DECLSPEC void SDL_UpdateRects
+extern DECLSPEC void SDLCALL SDL_UpdateRects
 		(SDL_Surface *screen, int numrects, SDL_Rect *rects);
-extern DECLSPEC void SDL_UpdateRect
+extern DECLSPEC void SDLCALL SDL_UpdateRect
 		(SDL_Surface *screen, Sint32 x, Sint32 y, Uint32 w, Uint32 h);
 
 /*
@@ -349,7 +372,7 @@ extern DECLSPEC void SDL_UpdateRect
  * setting the video mode for this function to perform hardware flipping.
  * This function returns 0 if successful, or -1 if there was an error.
  */
-extern DECLSPEC int SDL_Flip(SDL_Surface *screen);
+extern DECLSPEC int SDLCALL SDL_Flip(SDL_Surface *screen);
 
 /*
  * Set the gamma correction for each of the color channels.
@@ -359,30 +382,32 @@ extern DECLSPEC int SDL_Flip(SDL_Surface *screen);
  * be emulated using gamma ramps, if available.  If successful, this
  * function returns 0, otherwise it returns -1.
  */
-extern DECLSPEC int SDL_SetGamma(float red, float green, float blue);
+extern DECLSPEC int SDLCALL SDL_SetGamma(float red, float green, float blue);
 
 /*
  * Set the gamma translation table for the red, green, and blue channels
- * of the video hardware.  Each table is an array of 256 8-bit quantities,
+ * of the video hardware.  Each table is an array of 256 16-bit quantities,
  * representing a mapping between the input and output for that channel.
+ * The input is the index into the array, and the output is the 16-bit
+ * gamma value at that index, scaled to the output color precision.
  * 
  * You may pass NULL for any of the channels to leave it unchanged.
  * If the call succeeds, it will return 0.  If the display driver or
  * hardware does not support gamma translation, or otherwise fails,
  * this function will return -1.
  */
-extern DECLSPEC int SDL_SetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue);
+extern DECLSPEC int SDLCALL SDL_SetGammaRamp(const Uint16 *red, const Uint16 *green, const Uint16 *blue);
 
 /*
  * Retrieve the current values of the gamma translation tables.
  * 
- * You must pass in valid pointers to arrays of 256 8-bit quantities.
+ * You must pass in valid pointers to arrays of 256 16-bit quantities.
  * Any of the pointers may be NULL to ignore that channel.
  * If the call succeeds, it will return 0.  If the display driver or
  * hardware does not support gamma translation, or otherwise fails,
  * this function will return -1.
  */
-extern DECLSPEC int SDL_GetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue);
+extern DECLSPEC int SDLCALL SDL_GetGammaRamp(Uint16 *red, Uint16 *green, Uint16 *blue);
 
 /*
  * Sets a portion of the colormap for the given 8-bit surface.  If 'surface'
@@ -399,7 +424,7 @@ extern DECLSPEC int SDL_GetGammaRamp(Uint8 *red, Uint8 *green, Uint8 *blue);
  * you desire, even if the window colormap has to be warped or run under
  * emulation.
  */
-extern DECLSPEC int SDL_SetColors(SDL_Surface *surface, 
+extern DECLSPEC int SDLCALL SDL_SetColors(SDL_Surface *surface, 
 			SDL_Color *colors, int firstcolor, int ncolors);
 
 /*
@@ -418,32 +443,32 @@ extern DECLSPEC int SDL_SetColors(SDL_Surface *surface,
  * SDL_SetColors() is equivalent to calling this function with
  *     flags = (SDL_LOGPAL|SDL_PHYSPAL).
  */
-extern DECLSPEC int SDL_SetPalette(SDL_Surface *surface, int flags,
+extern DECLSPEC int SDLCALL SDL_SetPalette(SDL_Surface *surface, int flags,
 				   SDL_Color *colors, int firstcolor,
 				   int ncolors);
 
 /*
  * Maps an RGB triple to an opaque pixel value for a given pixel format
  */
-extern DECLSPEC Uint32 SDL_MapRGB
+extern DECLSPEC Uint32 SDLCALL SDL_MapRGB
 			(SDL_PixelFormat *format, Uint8 r, Uint8 g, Uint8 b);
 
 /*
  * Maps an RGBA quadruple to a pixel value for a given pixel format
  */
-extern DECLSPEC Uint32 SDL_MapRGBA(SDL_PixelFormat *format,
+extern DECLSPEC Uint32 SDLCALL SDL_MapRGBA(SDL_PixelFormat *format,
 				   Uint8 r, Uint8 g, Uint8 b, Uint8 a);
 
 /*
  * Maps a pixel value into the RGB components for a given pixel format
  */
-extern DECLSPEC void SDL_GetRGB(Uint32 pixel, SDL_PixelFormat *fmt,
+extern DECLSPEC void SDLCALL SDL_GetRGB(Uint32 pixel, SDL_PixelFormat *fmt,
 				Uint8 *r, Uint8 *g, Uint8 *b);
 
 /*
  * Maps a pixel value into the RGBA components for a given pixel format
  */
-extern DECLSPEC void SDL_GetRGBA(Uint32 pixel, SDL_PixelFormat *fmt,
+extern DECLSPEC void SDLCALL SDL_GetRGBA(Uint32 pixel, SDL_PixelFormat *fmt,
 				 Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a);
 
 /*
@@ -481,13 +506,13 @@ extern DECLSPEC void SDL_GetRGBA(Uint32 pixel, SDL_PixelFormat *fmt,
  * the SDL_HWSURFACE flag set, and will be created in system memory instead.
  */
 #define SDL_AllocSurface    SDL_CreateRGBSurface
-extern DECLSPEC SDL_Surface *SDL_CreateRGBSurface
+extern DECLSPEC SDL_Surface * SDLCALL SDL_CreateRGBSurface
 			(Uint32 flags, int width, int height, int depth, 
 			Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
-extern DECLSPEC SDL_Surface *SDL_CreateRGBSurfaceFrom(void *pixels,
+extern DECLSPEC SDL_Surface * SDLCALL SDL_CreateRGBSurfaceFrom(void *pixels,
 			int width, int height, int depth, int pitch,
 			Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
-extern DECLSPEC void SDL_FreeSurface(SDL_Surface *surface);
+extern DECLSPEC void SDLCALL SDL_FreeSurface(SDL_Surface *surface);
 
 /*
  * SDL_LockSurface() sets up a surface for directly accessing the pixels.
@@ -507,8 +532,8 @@ extern DECLSPEC void SDL_FreeSurface(SDL_Surface *surface);
  *
  * SDL_LockSurface() returns 0, or -1 if the surface couldn't be locked.
  */
-extern DECLSPEC int SDL_LockSurface(SDL_Surface *surface);
-extern DECLSPEC void SDL_UnlockSurface(SDL_Surface *surface);
+extern DECLSPEC int SDLCALL SDL_LockSurface(SDL_Surface *surface);
+extern DECLSPEC void SDLCALL SDL_UnlockSurface(SDL_Surface *surface);
 
 /*
  * Load a surface from a seekable SDL data source (memory or file.)
@@ -516,7 +541,7 @@ extern DECLSPEC void SDL_UnlockSurface(SDL_Surface *surface);
  * Returns the new surface, or NULL if there was an error.
  * The new surface should be freed with SDL_FreeSurface().
  */
-extern DECLSPEC SDL_Surface * SDL_LoadBMP_RW(SDL_RWops *src, int freesrc);
+extern DECLSPEC SDL_Surface * SDLCALL SDL_LoadBMP_RW(SDL_RWops *src, int freesrc);
 
 /* Convenience macro -- load a surface from a file */
 #define SDL_LoadBMP(file)	SDL_LoadBMP_RW(SDL_RWFromFile(file, "rb"), 1)
@@ -526,7 +551,7 @@ extern DECLSPEC SDL_Surface * SDL_LoadBMP_RW(SDL_RWops *src, int freesrc);
  * If 'freedst' is non-zero, the source will be closed after being written.
  * Returns 0 if successful or -1 if there was an error.
  */
-extern DECLSPEC int SDL_SaveBMP_RW
+extern DECLSPEC int SDLCALL SDL_SaveBMP_RW
 		(SDL_Surface *surface, SDL_RWops *dst, int freedst);
 
 /* Convenience macro -- save a surface to a file */
@@ -542,7 +567,7 @@ extern DECLSPEC int SDL_SaveBMP_RW
  * If 'flag' is 0, this function clears any current color key.
  * This function returns 0, or -1 if there was an error.
  */
-extern DECLSPEC int SDL_SetColorKey
+extern DECLSPEC int SDLCALL SDL_SetColorKey
 			(SDL_Surface *surface, Uint32 flag, Uint32 key);
 
 /*
@@ -557,8 +582,10 @@ extern DECLSPEC int SDL_SetColorKey
  * If 'flag' is SDL_SRCALPHA, alpha blending is enabled for the surface.
  * OR:ing the flag with SDL_RLEACCEL requests RLE acceleration for the
  * surface; if SDL_RLEACCEL is not specified, the RLE accel will be removed.
+ *
+ * The 'alpha' parameter is ignored for surfaces that have an alpha channel.
  */
-extern DECLSPEC int SDL_SetAlpha(SDL_Surface *surface, Uint32 flag, Uint8 alpha);
+extern DECLSPEC int SDLCALL SDL_SetAlpha(SDL_Surface *surface, Uint32 flag, Uint8 alpha);
 
 /*
  * Sets the clipping rectangle for the destination surface in a blit.
@@ -572,7 +599,14 @@ extern DECLSPEC int SDL_SetAlpha(SDL_Surface *surface, Uint32 flag, Uint8 alpha)
  * Note that blits are automatically clipped to the edges of the source
  * and destination surfaces.
  */
-extern DECLSPEC SDL_bool SDL_SetClipRect(SDL_Surface *surface, SDL_Rect *rect);
+extern DECLSPEC SDL_bool SDLCALL SDL_SetClipRect(SDL_Surface *surface, const SDL_Rect *rect);
+
+/*
+ * Gets the clipping rectangle for the destination surface in a blit.
+ * 'rect' must be a pointer to a valid rectangle which will be filled
+ * with the correct values.
+ */
+extern DECLSPEC void SDLCALL SDL_GetClipRect(SDL_Surface *surface, SDL_Rect *rect);
 
 /*
  * Creates a new surface of the specified format, and then copies and maps 
@@ -580,11 +614,13 @@ extern DECLSPEC SDL_bool SDL_SetClipRect(SDL_Surface *surface, SDL_Rect *rect);
  * fast as possible.  If this function fails, it returns NULL.
  *
  * The 'flags' parameter is passed to SDL_CreateRGBSurface() and has those 
- * semantics.
+ * semantics.  You can also pass SDL_RLEACCEL in the flags parameter and
+ * SDL will try to RLE accelerate colorkey and alpha blits in the resulting
+ * surface.
  *
  * This function is used internally by SDL_DisplayFormat().
  */
-extern DECLSPEC SDL_Surface *SDL_ConvertSurface
+extern DECLSPEC SDL_Surface * SDLCALL SDL_ConvertSurface
 			(SDL_Surface *src, SDL_PixelFormat *fmt, Uint32 flags);
 
 /*
@@ -597,6 +633,49 @@ extern DECLSPEC SDL_Surface *SDL_ConvertSurface
  *
  * The blit function should not be called on a locked surface.
  *
+ * The blit semantics for surfaces with and without alpha and colorkey
+ * are defined as follows:
+ *
+ * RGBA->RGB:
+ *     SDL_SRCALPHA set:
+ * 	alpha-blend (using alpha-channel).
+ * 	SDL_SRCCOLORKEY ignored.
+ *     SDL_SRCALPHA not set:
+ * 	copy RGB.
+ * 	if SDL_SRCCOLORKEY set, only copy the pixels matching the
+ * 	RGB values of the source colour key, ignoring alpha in the
+ * 	comparison.
+ * 
+ * RGB->RGBA:
+ *     SDL_SRCALPHA set:
+ * 	alpha-blend (using the source per-surface alpha value);
+ * 	set destination alpha to opaque.
+ *     SDL_SRCALPHA not set:
+ * 	copy RGB, set destination alpha to source per-surface alpha value.
+ *     both:
+ * 	if SDL_SRCCOLORKEY set, only copy the pixels matching the
+ * 	source colour key.
+ * 
+ * RGBA->RGBA:
+ *     SDL_SRCALPHA set:
+ * 	alpha-blend (using the source alpha channel) the RGB values;
+ * 	leave destination alpha untouched. [Note: is this correct?]
+ * 	SDL_SRCCOLORKEY ignored.
+ *     SDL_SRCALPHA not set:
+ * 	copy all of RGBA to the destination.
+ * 	if SDL_SRCCOLORKEY set, only copy the pixels matching the
+ * 	RGB values of the source colour key, ignoring alpha in the
+ * 	comparison.
+ * 
+ * RGB->RGB: 
+ *     SDL_SRCALPHA set:
+ * 	alpha-blend (using the source per-surface alpha value).
+ *     SDL_SRCALPHA not set:
+ * 	copy RGB.
+ *     both:
+ * 	if SDL_SRCCOLORKEY set, only copy the pixels matching the
+ * 	source colour key.
+ *
  * If either of the surfaces were in video memory, and the blit returns -2,
  * the video memory was lost, so it should be reloaded with artwork and 
  * re-blitted:
@@ -618,13 +697,13 @@ extern DECLSPEC SDL_Surface *SDL_ConvertSurface
 /* This is the public blit function, SDL_BlitSurface(), and it performs
    rectangle validation and clipping before passing it to SDL_LowerBlit()
 */
-extern DECLSPEC int SDL_UpperBlit
+extern DECLSPEC int SDLCALL SDL_UpperBlit
 			(SDL_Surface *src, SDL_Rect *srcrect,
 			 SDL_Surface *dst, SDL_Rect *dstrect);
 /* This is a semi-private blit function and it performs low-level surface
    blitting only.
 */
-extern DECLSPEC int SDL_LowerBlit
+extern DECLSPEC int SDLCALL SDL_LowerBlit
 			(SDL_Surface *src, SDL_Rect *srcrect,
 			 SDL_Surface *dst, SDL_Rect *dstrect);
 
@@ -637,7 +716,7 @@ extern DECLSPEC int SDL_LowerBlit
  * can be generated by the SDL_MapRGB() function.
  * This function returns 0 on success, or -1 on error.
  */
-extern DECLSPEC int SDL_FillRect
+extern DECLSPEC int SDLCALL SDL_FillRect
 		(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color);
 
 /* 
@@ -651,7 +730,7 @@ extern DECLSPEC int SDL_FillRect
  *
  * If the conversion fails or runs out of memory, it returns NULL
  */
-extern DECLSPEC SDL_Surface * SDL_DisplayFormat(SDL_Surface *surface);
+extern DECLSPEC SDL_Surface * SDLCALL SDL_DisplayFormat(SDL_Surface *surface);
 
 /* 
  * This function takes a surface and copies it to a new surface of the
@@ -665,7 +744,7 @@ extern DECLSPEC SDL_Surface * SDL_DisplayFormat(SDL_Surface *surface);
  *
  * If the conversion fails or runs out of memory, it returns NULL
  */
-extern DECLSPEC SDL_Surface * SDL_DisplayFormatAlpha(SDL_Surface *surface);
+extern DECLSPEC SDL_Surface * SDLCALL SDL_DisplayFormatAlpha(SDL_Surface *surface);
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -677,12 +756,12 @@ extern DECLSPEC SDL_Surface * SDL_DisplayFormatAlpha(SDL_Surface *surface);
    the contents of the display surface underneath the area where the overlay
    is shown is undefined - it may be overwritten with the converted YUV data.
 */
-extern DECLSPEC SDL_Overlay *SDL_CreateYUVOverlay(int width, int height,
+extern DECLSPEC SDL_Overlay * SDLCALL SDL_CreateYUVOverlay(int width, int height,
 				Uint32 format, SDL_Surface *display);
 
 /* Lock an overlay for direct access, and unlock it when you are done */
-extern DECLSPEC int SDL_LockYUVOverlay(SDL_Overlay *overlay);
-extern DECLSPEC void SDL_UnlockYUVOverlay(SDL_Overlay *overlay);
+extern DECLSPEC int SDLCALL SDL_LockYUVOverlay(SDL_Overlay *overlay);
+extern DECLSPEC void SDLCALL SDL_UnlockYUVOverlay(SDL_Overlay *overlay);
 
 /* Blit a video overlay to the display surface.
    The contents of the video surface underneath the blit destination are
@@ -690,10 +769,10 @@ extern DECLSPEC void SDL_UnlockYUVOverlay(SDL_Overlay *overlay);
    The width and height of the destination rectangle may be different from
    that of the overlay, but currently only 2x scaling is supported.
 */
-extern DECLSPEC int SDL_DisplayYUVOverlay(SDL_Overlay *overlay, SDL_Rect *dstrect);
+extern DECLSPEC int SDLCALL SDL_DisplayYUVOverlay(SDL_Overlay *overlay, SDL_Rect *dstrect);
 
 /* Free a video overlay */
-extern DECLSPEC void SDL_FreeYUVOverlay(SDL_Overlay *overlay);
+extern DECLSPEC void SDLCALL SDL_FreeYUVOverlay(SDL_Overlay *overlay);
 
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -710,17 +789,17 @@ extern DECLSPEC void SDL_FreeYUVOverlay(SDL_Overlay *overlay);
  *
  * This is disabled in default builds of SDL.
  */
-extern DECLSPEC int SDL_GL_LoadLibrary(const char *path);
+extern DECLSPEC int SDLCALL SDL_GL_LoadLibrary(const char *path);
 
 /*
  * Get the address of a GL function (for extension functions)
  */
-extern DECLSPEC void *SDL_GL_GetProcAddress(const char* proc);
+extern DECLSPEC void * SDLCALL SDL_GL_GetProcAddress(const char* proc);
 
 /*
  * Set an attribute of the OpenGL subsystem before intialization.
  */
-extern DECLSPEC int SDL_GL_SetAttribute(SDL_GLattr attr, int value);
+extern DECLSPEC int SDLCALL SDL_GL_SetAttribute(SDL_GLattr attr, int value);
 
 /*
  * Get an attribute of the OpenGL subsystem from the windowing
@@ -731,16 +810,20 @@ extern DECLSPEC int SDL_GL_SetAttribute(SDL_GLattr attr, int value);
  * Developers should track the values they pass into SDL_GL_SetAttribute
  * themselves if they want to retrieve these values.
  */
-extern DECLSPEC int SDL_GL_GetAttribute(SDL_GLattr attr, int* value);
+extern DECLSPEC int SDLCALL SDL_GL_GetAttribute(SDL_GLattr attr, int* value);
 
 /*
  * Swap the OpenGL buffers, if double-buffering is supported.
  */
-extern DECLSPEC void SDL_GL_SwapBuffers(void);
+extern DECLSPEC void SDLCALL SDL_GL_SwapBuffers(void);
 
-extern DECLSPEC void SDL_GL_UpdateRects(int numrects, SDL_Rect* rects);
-extern DECLSPEC void SDL_GL_Lock(void);
-extern DECLSPEC void SDL_GL_Unlock(void);
+/*
+ * Internal functions that should not be called unless you have read
+ * and understood the source code for these functions.
+ */
+extern DECLSPEC void SDLCALL SDL_GL_UpdateRects(int numrects, SDL_Rect* rects);
+extern DECLSPEC void SDLCALL SDL_GL_Lock(void);
+extern DECLSPEC void SDLCALL SDL_GL_Unlock(void);
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* These functions allow interaction with the window manager, if any.        */
@@ -749,8 +832,8 @@ extern DECLSPEC void SDL_GL_Unlock(void);
 /*
  * Sets/Gets the title and icon text of the display window
  */
-extern DECLSPEC void SDL_WM_SetCaption(const char *title, const char *icon);
-extern DECLSPEC void SDL_WM_GetCaption(char **title, char **icon);
+extern DECLSPEC void SDLCALL SDL_WM_SetCaption(const char *title, const char *icon);
+extern DECLSPEC void SDLCALL SDL_WM_GetCaption(char **title, char **icon);
 
 /*
  * Sets the icon for the display window.
@@ -758,14 +841,14 @@ extern DECLSPEC void SDL_WM_GetCaption(char **title, char **icon);
  * It takes an icon surface, and a mask in MSB format.
  * If 'mask' is NULL, the entire icon surface will be used as the icon.
  */
-extern DECLSPEC void SDL_WM_SetIcon(SDL_Surface *icon, Uint8 *mask);
+extern DECLSPEC void SDLCALL SDL_WM_SetIcon(SDL_Surface *icon, Uint8 *mask);
 
 /*
  * This function iconifies the window, and returns 1 if it succeeded.
  * If the function succeeds, it generates an SDL_APPACTIVE loss event.
  * This function is a noop and returns 0 in non-windowed environments.
  */
-extern DECLSPEC int SDL_WM_IconifyWindow(void);
+extern DECLSPEC int SDLCALL SDL_WM_IconifyWindow(void);
 
 /*
  * Toggle fullscreen mode without changing the contents of the screen.
@@ -782,7 +865,7 @@ extern DECLSPEC int SDL_WM_IconifyWindow(void);
  *
  * This is currently only implemented in the X11 video driver.
  */
-extern DECLSPEC int SDL_WM_ToggleFullScreen(SDL_Surface *surface);
+extern DECLSPEC int SDLCALL SDL_WM_ToggleFullScreen(SDL_Surface *surface);
 
 /*
  * This function allows you to set and query the input grab state of
@@ -799,11 +882,15 @@ typedef enum {
  * and nearly all keyboard input is passed directly to the application,
  * and not interpreted by a window manager, if any.
  */
-extern DECLSPEC SDL_GrabMode SDL_WM_GrabInput(SDL_GrabMode mode);
+extern DECLSPEC SDL_GrabMode SDLCALL SDL_WM_GrabInput(SDL_GrabMode mode);
 
+/* Not in public API at the moment - do not use! */
+extern DECLSPEC int SDLCALL SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
+                                    SDL_Surface *dst, SDL_Rect *dstrect);
+                    
 /* Ends C function definitions when using C++ */
 #ifdef __cplusplus
-};
+}
 #endif
 #include "close_code.h"
 
