@@ -22,12 +22,15 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL.c,v 1.5 2002/03/06 11:23:02 slouken Exp $";
+ "@(#) $Id: SDL.c,v 1.8 2002/12/07 06:54:47 slouken Exp $";
 #endif
 
 /* Initialization code for SDL */
 
 #include <stdlib.h>		/* For getenv() */
+#ifdef ENABLE_PTH
+#include <pth.h>
+#endif
 
 #include "SDL.h"
 #include "SDL_endian.h"
@@ -150,6 +153,12 @@ int SDL_InitSubSystem(Uint32 flags)
 
 int SDL_Init(Uint32 flags)
 {
+#if !defined(DISABLE_THREADS) && defined(ENABLE_PTH)
+	if (!pth_init()) {
+		return -1;
+	}
+#endif
+
 	/* Clear the error message */
 	SDL_ClearError();
 
@@ -223,6 +232,10 @@ void SDL_Quit(void)
 
 	/* Uninstall any parachute signal handlers */
 	SDL_UninstallParachute();
+
+#if !defined(DISABLE_THREADS) && defined(ENABLE_PTH)
+	pth_kill();
+#endif
 }
 
 /* Return the library version number */
@@ -231,8 +244,8 @@ const SDL_version * SDL_Linked_Version(void)
 	return(&version);
 }
 
-#if defined(_WIN32_WCE)
-/* Need to include DllMain() on Windows CE for some reason.. */
+#if defined(_WIN32_WCE) || (defined(__WATCOMC__) && defined(BUILD_DLL))
+/* Need to include DllMain() on Windows CE and Watcom C for some reason.. */
 #include <windows.h>
 
 BOOL APIENTRY DllMain( HANDLE hModule, 
@@ -248,4 +261,4 @@ BOOL APIENTRY DllMain( HANDLE hModule,
 	}
 	return TRUE;
 }
-#endif /* _WIN32_WCE */
+#endif /* _WIN32_WCE and building DLL with Watcom C */
