@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_audio.c,v 1.10 2001/08/09 13:09:47 slouken Exp $";
+ "@(#) $Id: SDL_audio.c,v 1.20 2003/08/04 00:52:42 slouken Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -50,6 +50,9 @@ static AudioBootStrap *bootstrap[] = {
 #ifdef ALSA_SUPPORT
 	&ALSA_bootstrap,
 #endif
+#ifdef QNXNTOAUDIO_SUPPORT
+	&QNXNTOAUDIO_bootstrap,
+#endif
 #ifdef SUNAUDIO_SUPPORT
 	&SUNAUDIO_bootstrap,
 #endif
@@ -83,8 +86,24 @@ static AudioBootStrap *bootstrap[] = {
 #ifdef ENABLE_AHI
 	&AHI_bootstrap,
 #endif
+#ifdef MMEAUDIO_SUPPORT
+	&MMEAUDIO_bootstrap,
+#endif
+#ifdef MINTAUDIO_SUPPORT
+	&MINTAUDIO_GSXB_bootstrap,
+	&MINTAUDIO_MCSN_bootstrap,
+	&MINTAUDIO_STFA_bootstrap,
+	&MINTAUDIO_XBIOS_bootstrap,
+	&MINTAUDIO_DMA8_bootstrap,
+#endif
 #ifdef DISKAUD_SUPPORT
 	&DISKAUD_bootstrap,
+#endif
+#ifdef ENABLE_DC
+	&DCAUD_bootstrap,
+#endif
+#ifdef DRENDERER_SUPPORT
+	&DRENDERER_bootstrap,
 #endif
 	NULL
 };
@@ -230,6 +249,22 @@ int SDL_RunAudio(void *audiop)
 	return(0);
 }
 
+static void SDL_LockAudio_Default(SDL_AudioDevice *audio)
+{
+	if ( audio->thread && (SDL_ThreadID() == audio->threadid) ) {
+		return;
+	}
+	SDL_mutexP(audio->mixer_lock);
+}
+
+static void SDL_UnlockAudio_Default(SDL_AudioDevice *audio)
+{
+	if ( audio->thread && (SDL_ThreadID() == audio->threadid) ) {
+		return;
+	}
+	SDL_mutexV(audio->mixer_lock);
+}
+
 int SDL_AudioInit(const char *driver_name)
 {
 	SDL_AudioDevice *audio;
@@ -309,6 +344,10 @@ int SDL_AudioInit(const char *driver_name)
 	current_audio = audio;
 	if ( current_audio ) {
 		current_audio->name = bootstrap[i]->name;
+		if ( !current_audio->LockAudio && !current_audio->UnlockAudio ) {
+			current_audio->LockAudio = SDL_LockAudio_Default;
+			current_audio->UnlockAudio = SDL_UnlockAudio_Default;
+		}
 	}
 	return(0);
 }
@@ -336,6 +375,11 @@ int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 	}
 	audio = current_audio;
 
+	if (audio->opened) {
+		SDL_SetError("Audio device is already opened");
+		return(-1);
+	}
+
 	/* Verify some parameters */
 	if ( desired->callback == NULL ) {
 		SDL_SetError("SDL_OpenAudio() passed a NULL callback");
@@ -350,8 +394,11 @@ int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 		return(-1);
 	}
 
-#ifdef macintosh
+#if defined(macintosh) || defined(__riscos__)
 	/* FIXME: Need to implement PPC interrupt asm for SDL_LockAudio() */
+#else
+#if defined(__MINT__) && !defined(ENABLE_THREADS)
+	/* Uses interrupt driven audio, without thread */
 #else
 	/* Create a semaphore for locking the sound buffers */
 	audio->mixer_lock = SDL_CreateMutex();
@@ -360,7 +407,8 @@ int SDL_OpenAudio(SDL_AudioSpec *desired, SDL_AudioSpec *obtained)
 		SDL_CloseAudio();
 		return(-1);
 	}
-#endif
+#endif /* __MINT__ */
+#endif /* macintosh */
 
 	/* Calculate the silence and size of the audio specification */
 	SDL_CalculateAudioSpec(desired);
@@ -501,11 +549,8 @@ void SDL_LockAudio (void)
 	SDL_AudioDevice *audio = current_audio;
 
 	/* Obtain a lock on the mixing buffers */
-	if ( audio ) {
-		if ( audio->thread && (SDL_ThreadID() == audio->threadid) ) {
-			return;
-		}
-		SDL_mutexP(audio->mixer_lock);
+	if ( audio && audio->LockAudio ) {
+		audio->LockAudio(audio);
 	}
 }
 
@@ -514,11 +559,8 @@ void SDL_UnlockAudio (void)
 	SDL_AudioDevice *audio = current_audio;
 
 	/* Release lock on the mixing buffers */
-	if ( audio ) {
-		if ( audio->thread && (SDL_ThreadID() == audio->threadid) ) {
-			return;
-		}
-		SDL_mutexV(audio->mixer_lock);
+	if ( audio && audio->UnlockAudio ) {
+		audio->UnlockAudio(audio);
 	}
 }
 
