@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_mixer.c,v 1.4 2002/03/06 11:23:02 slouken Exp $";
+ "@(#) $Id: SDL_mixer.c,v 1.7 2003/06/03 19:35:10 pmandin Exp $";
 #endif
 
 /* This provides the default mixing callback for the SDL audio routines */
@@ -35,7 +35,26 @@ static char rcsid =
 #include "SDL_mutex.h"
 #include "SDL_timer.h"
 #include "SDL_sysaudio.h"
+#include "SDL_mixer_MMX.h"
+#include "SDL_mixer_MMX_VC.h"
+#include "SDL_mixer_m68k.h"
 
+/* Function to check the CPU flags */
+#define MMX_CPU		0x800000
+#ifdef USE_ASMBLIT
+#define CPU_Flags()	Hermes_X86_CPU()
+#else
+#define CPU_Flags()	0L
+#endif
+
+#ifdef USE_ASMBLIT
+#define X86_ASSEMBLER
+#define HermesConverterInterface	void
+#define HermesClearInterface		void
+#define STACKCALL
+
+#include "HeadX86.h"
+#endif
 
 /* This table is used to add two sound values together and pin
  * the value to avoid overflow.  (used with permission from ARDI)
@@ -117,6 +136,9 @@ void SDL_MixAudio (Uint8 *dst, const Uint8 *src, Uint32 len, int volume)
 	switch (format) {
 
 		case AUDIO_U8: {
+#if defined(__M68000__) && defined(__GNUC__)
+			SDL_MixAudio_m68k_U8((char*)dst,(char*)src,(unsigned long)len,(long)volume,(char *)mix8);
+#else
 			Uint8 src_sample;
 
 			while ( len-- ) {
@@ -126,10 +148,29 @@ void SDL_MixAudio (Uint8 *dst, const Uint8 *src, Uint32 len, int volume)
 				++dst;
 				++src;
 			}
+#endif
 		}
 		break;
 
 		case AUDIO_S8: {
+#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
+			if (CPU_Flags() & MMX_CPU)
+			{
+				SDL_MixAudio_MMX_S8((char*)dst,(char*)src,(unsigned int)len,(int)volume);
+			}
+			else
+#endif
+#if defined(USE_ASM_MIXER_VC)
+			if (SDL_IsMMX_VC())
+			{
+				SDL_MixAudio_MMX_S8_VC((char*)dst,(char*)src,(unsigned int)len,(int)volume);
+			}
+			else
+#endif
+#if defined(__M68000__) && defined(__GNUC__)
+			SDL_MixAudio_m68k_S8((char*)dst,(char*)src,(unsigned long)len,(long)volume);
+#else
+			{
 			Sint8 *dst8, *src8;
 			Sint8 src_sample;
 			int dst_sample;
@@ -153,10 +194,29 @@ void SDL_MixAudio (Uint8 *dst, const Uint8 *src, Uint32 len, int volume)
 				++dst8;
 				++src8;
 			}
+			}
+#endif
 		}
 		break;
 
 		case AUDIO_S16LSB: {
+#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
+			if (CPU_Flags() & MMX_CPU)
+			{
+				SDL_MixAudio_MMX_S16((char*)dst,(char*)src,(unsigned int)len,(int)volume);
+			}
+			else
+#elif defined(USE_ASM_MIXER_VC)
+			if (SDL_IsMMX_VC())
+			{
+				SDL_MixAudio_MMX_S16_VC((char*)dst,(char*)src,(unsigned int)len,(int)volume);
+			}
+			else
+#endif
+#if defined(__M68000__) && defined(__GNUC__)
+			SDL_MixAudio_m68k_S16LSB((short*)dst,(short*)src,(unsigned long)len,(long)volume);
+#else
+			{
 			Sint16 src1, src2;
 			int dst_sample;
 			const int max_audioval = ((1<<(16-1))-1);
@@ -180,10 +240,15 @@ void SDL_MixAudio (Uint8 *dst, const Uint8 *src, Uint32 len, int volume)
 				dst[1] = dst_sample&0xFF;
 				dst += 2;
 			}
+			}
+#endif
 		}
 		break;
 
 		case AUDIO_S16MSB: {
+#if defined(__M68000__) && defined(__GNUC__)
+			SDL_MixAudio_m68k_S16MSB((short*)dst,(short*)src,(unsigned long)len,(long)volume);
+#else
 			Sint16 src1, src2;
 			int dst_sample;
 			const int max_audioval = ((1<<(16-1))-1);
@@ -207,6 +272,7 @@ void SDL_MixAudio (Uint8 *dst, const Uint8 *src, Uint32 len, int volume)
 				dst[0] = dst_sample&0xFF;
 				dst += 2;
 			}
+#endif
 		}
 		break;
 
@@ -215,3 +281,4 @@ void SDL_MixAudio (Uint8 *dst, const Uint8 *src, Uint32 len, int volume)
 			return;
 	}
 }
+
