@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_artsaudio.c,v 1.2 2001/04/26 16:50:17 hercules Exp $";
+ "@(#) $Id: SDL_artsaudio.c,v 1.7 2002/08/28 23:25:09 slouken Exp $";
 #endif
 
 /* Allow access to a raw mixing buffer */
@@ -43,6 +43,13 @@ static char rcsid =
 #include "SDL_audiodev_c.h"
 #include "SDL_artsaudio.h"
 
+#ifdef ARTSC_DYNAMIC
+#include "SDL_name.h"
+#include "SDL_loadso.h"
+#else
+#define SDL_NAME(X)	X
+#endif
+
 /* The tag name used by artsc audio */
 #define ARTSC_DRIVER_NAME         "artsc"
 
@@ -53,20 +60,99 @@ static void ARTSC_PlayAudio(_THIS);
 static Uint8 *ARTSC_GetAudioBuf(_THIS);
 static void ARTSC_CloseAudio(_THIS);
 
+#ifdef ARTSC_DYNAMIC
+
+static const char *arts_library = ARTSC_DYNAMIC;
+static void *arts_handle = NULL;
+static int arts_loaded = 0;
+
+static int (*SDL_NAME(arts_init))(void);
+static void (*SDL_NAME(arts_free))(void);
+static arts_stream_t (*SDL_NAME(arts_play_stream))(int rate, int bits, int channels, const char *name);
+static int (*SDL_NAME(arts_stream_set))(arts_stream_t s, arts_parameter_t param, int value);
+static int (*SDL_NAME(arts_stream_get))(arts_stream_t s, arts_parameter_t param);
+static int (*SDL_NAME(arts_write))(arts_stream_t s, const void *buffer, int count);
+static void (*SDL_NAME(arts_close_stream))(arts_stream_t s);
+
+static struct {
+	const char *name;
+	void **func;
+} arts_functions[] = {
+	{ "arts_init",		(void **)&SDL_NAME(arts_init)		},
+	{ "arts_free",		(void **)&SDL_NAME(arts_free)		},
+	{ "arts_play_stream",	(void **)&SDL_NAME(arts_play_stream)	},
+	{ "arts_stream_set",	(void **)&SDL_NAME(arts_stream_set)	},
+	{ "arts_stream_get",	(void **)&SDL_NAME(arts_stream_get)	},
+	{ "arts_write",		(void **)&SDL_NAME(arts_write)		},
+	{ "arts_close_stream",	(void **)&SDL_NAME(arts_close_stream)	},
+};
+
+static void UnloadARTSLibrary()
+{
+	if ( arts_loaded ) {
+		SDL_UnloadObject(arts_handle);
+		arts_handle = NULL;
+		arts_loaded = 0;
+	}
+}
+
+static int LoadARTSLibrary(void)
+{
+	int i, retval = -1;
+
+	arts_handle = SDL_LoadObject(arts_library);
+	if ( arts_handle ) {
+		arts_loaded = 1;
+		retval = 0;
+		for ( i=0; i<SDL_TABLESIZE(arts_functions); ++i ) {
+			*arts_functions[i].func = SDL_LoadFunction(arts_handle, arts_functions[i].name);
+			if ( ! arts_functions[i].func ) {
+				retval = -1;
+				UnloadARTSLibrary();
+				break;
+			}
+		}
+	}
+	return retval;
+}
+
+#else
+
+static void UnloadARTSLibrary()
+{
+	return;
+}
+
+static int LoadARTSLibrary(void)
+{
+	return 0;
+}
+
+#endif /* ARTSC_DYNAMIC */
+
 /* Audio driver bootstrap functions */
 
 static int Audio_Available(void)
 {
-	if(arts_init())
-		return 0;
-	else
-		return 1;
+	int available = 0;
+
+	if ( LoadARTSLibrary() < 0 ) {
+		return available;
+	}
+	if ( SDL_NAME(arts_init)() == 0 ) {
+		available = 1;
+		SDL_NAME(arts_free)();
+	}
+	UnloadARTSLibrary();
+
+	return available;
 }
 
 static void Audio_DeleteDevice(SDL_AudioDevice *device)
 {
 	free(device->hidden);
 	free(device);
+	UnloadARTSLibrary();
 }
 
 static SDL_AudioDevice *Audio_CreateDevice(int devindex)
@@ -74,6 +160,7 @@ static SDL_AudioDevice *Audio_CreateDevice(int devindex)
 	SDL_AudioDevice *this;
 
 	/* Initialize all variables that we clean on shutdown */
+	LoadARTSLibrary();
 	this = (SDL_AudioDevice *)malloc(sizeof(SDL_AudioDevice));
 	if ( this ) {
 		memset(this, 0, (sizeof *this));
@@ -136,7 +223,7 @@ static void ARTSC_PlayAudio(_THIS)
 	int written;
 
 	/* Write the audio data */
-	written = arts_write(stream, mixbuf, mixlen);
+	written = SDL_NAME(arts_write)(stream, mixbuf, mixlen);
 	
 	/* If timer synchronization is enabled, set the next write frame */
 	if ( frame_ticks ) {
@@ -164,9 +251,10 @@ static void ARTSC_CloseAudio(_THIS)
 		mixbuf = NULL;
 	}
 	if ( stream ) {
-		arts_close_stream(stream);
+		SDL_NAME(arts_close_stream)(stream);
 		stream = 0;
 	}
+	SDL_NAME(arts_free)();
 }
 
 static int ARTSC_OpenAudio(_THIS, SDL_AudioSpec *spec)
@@ -210,7 +298,11 @@ static int ARTSC_OpenAudio(_THIS, SDL_AudioSpec *spec)
 	}
 	spec->format = test_format;
 
-	stream = arts_play_stream(spec->freq, bits, spec->channels, "SDL");
+	if ( SDL_NAME(arts_init)() != 0 ) {
+		SDL_SetError("Unable to initialize ARTS");
+		return(-1);
+	}
+	stream = SDL_NAME(arts_play_stream)(spec->freq, bits, spec->channels, "SDL");
 
 	/* Calculate the final parameters for this audio specification */
 	SDL_CalculateAudioSpec(spec);
@@ -224,12 +316,12 @@ static int ARTSC_OpenAudio(_THIS, SDL_AudioSpec *spec)
 	frag_spec |= 0x00020000;	/* two fragments, for low latency */
 
 #ifdef ARTS_P_PACKET_SETTINGS
-	arts_stream_set(stream, ARTS_P_PACKET_SETTINGS, frag_spec);
+	SDL_NAME(arts_stream_set)(stream, ARTS_P_PACKET_SETTINGS, frag_spec);
 #else
-	arts_stream_set(stream, ARTS_P_PACKET_SIZE, frag_spec&0xffff);
-	arts_stream_set(stream, ARTS_P_PACKET_COUNT, frag_spec>>16);
+	SDL_NAME(arts_stream_set)(stream, ARTS_P_PACKET_SIZE, frag_spec&0xffff);
+	SDL_NAME(arts_stream_set)(stream, ARTS_P_PACKET_COUNT, frag_spec>>16);
 #endif
-	spec->size = arts_stream_get(stream, ARTS_P_PACKET_SIZE);
+	spec->size = SDL_NAME(arts_stream_get)(stream, ARTS_P_PACKET_SIZE);
 
 	/* Allocate mixing buffer */
 	mixlen = spec->size;
