@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 
     This driver was written by:
     Erik Inge Bols
@@ -26,13 +26,17 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_nasaudio.h,v 1.2 2001/04/26 16:50:17 hercules Exp $";
+ "@(#) $Id: SDL_nasaudio.h,v 1.5 2003/03/06 06:12:41 slouken Exp $";
 #endif
 
 #ifndef _SDL_nasaudio_h
 #define _SDL_nasaudio_h
 
+#ifdef __sgi
+#include <nas/audiolib.h>
+#else
 #include <audio/audiolib.h>
+#endif
 #include <sys/time.h>
 #include "SDL_sysaudio.h"
 
