@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,7 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 /*
@@ -304,7 +304,7 @@ audio mode	:   %s
     (info.mode == AUMODE_PLAY) ? "PLAY"
     : (info.mode = AUMODE_RECORD) ? "RECORD"
     : (info.mode == AUMODE_PLAY_ALL ? "PLAY_ALL"
-    : "???"));
+    : "?"));
 }
 #endif /* DEBUG_AUDIO */
 
