@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_mouse.c,v 1.4 2002/03/06 11:23:02 slouken Exp $";
+ "@(#) $Id: SDL_mouse.c,v 1.6 2002/10/07 15:19:34 slouken Exp $";
 #endif
 
 /* General mouse handling code for SDL */
@@ -59,6 +59,17 @@ int SDL_MouseInit(void)
 	return(0);
 }
 
+/* We lost the mouse, so post button up messages for all pressed buttons */
+void SDL_ResetMouse(void)
+{
+	Uint8 i;
+	for ( i = 0; i < sizeof(SDL_ButtonState)*8; ++i ) {
+		if ( SDL_ButtonState & SDL_BUTTON(i) ) {
+			SDL_PrivateMouseButton(SDL_RELEASED, i, 0, 0);
+		}
+	}
+}
+
 Uint8 SDL_GetMouseState (int *x, int *y)
 {
 	if ( x )
