@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysjoystick.c,v 1.4 2001/11/02 18:12:52 slouken Exp $";
+ "@(#) $Id: SDL_sysjoystick.c,v 1.12 2003/02/01 20:25:34 slouken Exp $";
 #endif
 
 /* This is the system specific header for the SDL joystick API */
@@ -35,7 +35,9 @@ static char rcsid =
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <limits.h>		/* For the definition of PATH_MAX */
-
+#ifdef __arm__
+#include <linux/limits.h> /* Arm cross-compiler needs this */
+#endif
 #include <linux/joystick.h>
 #ifdef USE_INPUT_EVENTS
 #include <linux/input.h>
@@ -46,30 +48,24 @@ static char rcsid =
 #include "SDL_sysjoystick.h"
 #include "SDL_joystick_c.h"
 
-/* Define this if you want to map axes to hats and trackballs */
-#define FANCY_HATS_AND_BALLS
-
-#ifdef FANCY_HATS_AND_BALLS
-/* Special joystick configurations:
-	'JoystickName' Naxes Nhats Nballs
- */
-static const char *special_joysticks[] = {
-	"'MadCatz Panther XL' 3 2 1", /* We don't handle a rudder (axis 8) */
-	"'SideWinder Precision Pro' 4 1 0",
-	"'SideWinder 3D Pro' 4 1 0",
-	"'Microsoft SideWinder 3D Pro' 4 1 0",
-	"'Microsoft SideWinder Dual Strike USB version 1.0' 2 1 0",
-	"'WingMan Interceptor' 3 3 0",
-	/* WingMan Extreme Analog - not recognized by default
-	"'Analog 3-axis 4-button joystick' 2 1 0",
-	*/
-	"'WingMan Extreme Digital 3D' 4 1 0",
-	"'Analog 2-axis 4-button 1-hat FCS joystick' 2 1 0",
-	NULL
+/* Special joystick configurations */
+static struct {
+	const char *name;
+	int naxes;
+	int nhats;
+	int nballs;
+} special_joysticks[] = {
+	{ "MadCatz Panther XL", 3, 2, 1 }, /* We don't handle rudder (axis 8) */
+	{ "SideWinder Precision Pro", 4, 1, 0 },
+	{ "SideWinder 3D Pro", 4, 1, 0 },
+	{ "Microsoft SideWinder 3D Pro", 4, 1, 0 },
+	{ "Microsoft SideWinder Dual Strike USB version 1.0", 2, 1, 0 },
+	{ "WingMan Interceptor", 3, 3, 0 },
+	{ "WingMan Extreme Digital 3D", 4, 1, 0 },
+	{ "Microsoft SideWinder Precision 2 Joystick", 4, 1, 0 },
+	{ "Logitech Inc. WingMan Extreme Digital 3D", 4, 1, 0 },
+	{ "Saitek Saitek X45", 6, 1, 0 }
 };
-#else
-#undef USE_INPUT_EVENTS
-#endif
 
 /* The maximum number of joysticks we'll detect */
 #define MAX_JOYSTICKS	32
@@ -81,7 +77,6 @@ static char *SDL_joylist[MAX_JOYSTICKS];
 struct joystick_hwdata {
 	int fd;
 	/* The current linux joystick driver maps hats to two axes */
-	int analog_hat;		/* Well, except for analog hats */
 	struct hwdata_hat {
 		int axis[2];
 	} *hats;
@@ -91,8 +86,8 @@ struct joystick_hwdata {
 	} *balls;
 
 	/* Support for the Linux 2.4 unified input interface */
-	SDL_bool is_hid;
 #ifdef USE_INPUT_EVENTS
+	SDL_bool is_hid;
 	Uint8 key_map[KEY_MAX-BTN_MISC];
 	Uint8 abs_map[ABS_MAX];
 	struct axis_correct {
@@ -140,15 +135,15 @@ static int EV_IsJoystick(int fd)
 int SDL_SYS_JoystickInit(void)
 {
 	/* The base path of the joystick devices */
-	const char *joydev_pattern[2] = {
-		"/dev/js%d",
+	const char *joydev_pattern[] = {
 #ifdef USE_INPUT_EVENTS
-		"/dev/input/event%d"
+		"/dev/input/event%d",
 #endif
-		"/dev/input/js%d"
+		"/dev/input/js%d",
+		"/dev/js%d"
 	};
 	int numjoysticks;
-	int i, j, done;
+	int i, j;
 	int fd;
 	char path[PATH_MAX];
 	dev_t dev_nums[MAX_JOYSTICKS];  /* major/minor device numbers */
@@ -174,9 +169,9 @@ int SDL_SYS_JoystickInit(void)
 			}
 		}
 	}
+
 	for ( i=0; i<SDL_TABLESIZE(joydev_pattern); ++i ) {
-		done = 0;
-		for ( j=0; (j < MAX_JOYSTICKS) && !done; ++j ) {
+		for ( j=0; j < MAX_JOYSTICKS; ++j ) {
 			sprintf(path, joydev_pattern[i], j);
 
 			/* rcg06302000 replaced access(F_OK) call with stat().
@@ -205,7 +200,7 @@ int SDL_SYS_JoystickInit(void)
 #ifdef DEBUG_INPUT_EVENTS
 				printf("Checking %s\n", path);
 #endif
-				if ( (i > 0) && ! EV_IsJoystick(fd) ) {
+				if ( (i == 0) && ! EV_IsJoystick(fd) ) {
 					close(fd);
 					continue;
 				}
@@ -218,20 +213,23 @@ int SDL_SYS_JoystickInit(void)
 					dev_nums[numjoysticks] = sb.st_rdev;
 					++numjoysticks;
 				}
-			} else {
-				done = 1;
-			}
+			} else
+				break;
 		}
-        /* This is a special case...
-           If we're looking at the /dev/input event devices, and we found
-           at least one, then we don't want to look at the input joystick
-           devices, since they're built on top of devices that we've already
-           seen, so we're done.
-         */
-        if ( i > 0 && j > 0 ) {
-            done = 1;
-        }
+
+#ifdef USE_INPUT_EVENTS
+		/* This is a special case...
+		   If the event devices are valid then the joystick devices
+		   will be duplicates but without extra information about their
+		   hats or balls. Unfortunately, the event devices can't
+		   currently be calibrated, so it's a win-lose situation.
+		   So : /dev/input/eventX = /dev/input/jsY = /dev/jsY
+		*/
+		if ( (i == 0) && (numjoysticks > 0) )
+			break;
+#endif
 	}
+
 	return(numjoysticks);
 }
 
@@ -259,8 +257,6 @@ const char *SDL_SYS_JoystickName(int index)
 	return name;
 }
 
-#ifdef FANCY_HATS_AND_BALLS
-
 static int allocate_hatdata(SDL_Joystick *joystick)
 {
 	int i;
@@ -293,40 +289,84 @@ static int allocate_balldata(SDL_Joystick *joystick)
 	return(0);
 }
 
-static SDL_bool ConfigJoystick(SDL_Joystick *joystick,
-			const char *name, const char *config)
+static SDL_bool JS_ConfigJoystick(SDL_Joystick *joystick, int fd)
 {
-	char cfg_name[128];
 	SDL_bool handled;
+	unsigned char n;
+	int old_axes, tmp_naxes, tmp_nhats, tmp_nballs;
+	const char *name;
+	char *env, env_name[128];
+	int i;
 
-	if ( config == NULL ) {
-		return(SDL_FALSE);
-	}
-	strcpy(cfg_name, "");
-	if ( *config == '\'' ) {
-		sscanf(config, "'%[^']s'", cfg_name);
-		config += strlen(cfg_name)+2;
-	} else {
-		sscanf(config, "%s", cfg_name);
-		config += strlen(cfg_name);
-	}
 	handled = SDL_FALSE;
-	if ( strcmp(cfg_name, name) == 0 ) {
-		/* Get the number of axes, hats and balls for this joystick */
-		int joystick_axes = joystick->naxes;
-		sscanf(config, "%d %d %d", 
-			&joystick->naxes, &joystick->nhats, &joystick->nballs);
 
-		/* Allocate the extra data for mapping them */
+	/* Default joystick device settings */
+	if ( ioctl(fd, JSIOCGAXES, &n) < 0 ) {
+		joystick->naxes = 2;
+	} else {
+		joystick->naxes = n;
+	}
+	if ( ioctl(fd, JSIOCGBUTTONS, &n) < 0 ) {
+		joystick->nbuttons = 2;
+	} else {
+		joystick->nbuttons = n;
+	}
+
+	name = SDL_SYS_JoystickName(joystick->index);
+	old_axes = joystick->naxes;
+
+	/* Generic analog joystick support */
+	if ( strstr(name, "Analog") == name && strstr(name, "-hat") ) {
+		if ( sscanf(name,"Analog %d-axis %*d-button %d-hat",
+			&tmp_naxes, &tmp_nhats) == 2 ) {
+
+			joystick->naxes = tmp_naxes;
+			joystick->nhats = tmp_nhats;
+
+			handled = SDL_TRUE;
+		}
+	}
+
+	/* Special joystick support */
+	for ( i=0; i < SDL_TABLESIZE(special_joysticks); ++i ) {
+		if ( strcmp(name, special_joysticks[i].name) == 0 ) {
+
+			joystick->naxes = special_joysticks[i].naxes;
+			joystick->nhats = special_joysticks[i].nhats;
+			joystick->nballs = special_joysticks[i].nballs;
+
+			handled = SDL_TRUE;
+			break;
+		}
+	}
+
+	/* User environment joystick support */
+	if ( (env = getenv("SDL_LINUX_JOYSTICK")) ) {
+		strcpy(env_name, "");
+		if ( *env == '\'' && sscanf(env, "'%[^']s'", env_name) == 1 )
+			env += strlen(env_name)+2;
+		else if ( sscanf(env, "%s", env_name) == 1 )
+			env += strlen(env_name);
+
+		if ( strcmp(name, env_name) == 0 ) {
+
+			if ( sscanf(env, "%d %d %d", &tmp_naxes, &tmp_nhats,
+				&tmp_nballs) == 3 ) {
+
+				joystick->naxes = tmp_naxes;
+				joystick->nhats = tmp_nhats;
+				joystick->nballs = tmp_nballs;
+
+				handled = SDL_TRUE;
+			}
+		}
+	}
+
+	/* Remap hats and balls */
+	if (handled) {
 		if ( joystick->nhats > 0 ) {
-			/* HACK: Analog hats map to only one axis */
-			if (joystick_axes == (joystick->naxes+joystick->nhats)){
-				joystick->hwdata->analog_hat = 1;
-			} else {
-				if ( allocate_hatdata(joystick) < 0 ) {
-					joystick->nhats = 0;
-				}
-				joystick->hwdata->analog_hat = 0;
+			if ( allocate_hatdata(joystick) < 0 ) {
+				joystick->nhats = 0;
 			}
 		}
 		if ( joystick->nballs > 0 ) {
@@ -334,8 +374,8 @@ static SDL_bool ConfigJoystick(SDL_Joystick *joystick,
 				joystick->nballs = 0;
 			}
 		}
-		handled = SDL_TRUE;
 	}
+
 	return(handled);
 }
 
@@ -435,8 +475,6 @@ static SDL_bool EV_ConfigJoystick(SDL_Joystick *joystick, int fd)
 
 #endif /* USE_INPUT_EVENTS */
 
-#endif /* FANCY_HATS_AND_BALLS */
-
 /* Function to open a joystick for use.
    The joystick to open is specified by the index field of the joystick.
    This should fill the nbuttons and naxes fields of the joystick structure.
@@ -444,12 +482,7 @@ static SDL_bool EV_ConfigJoystick(SDL_Joystick *joystick, int fd)
  */
 int SDL_SYS_JoystickOpen(SDL_Joystick *joystick)
 {
-#ifdef FANCY_HATS_AND_BALLS
-	const char *name;
-	int i;
-#endif
 	int fd;
-	unsigned char n;
 
 	/* Open the joystick and set the joystick file descriptor */
 	fd = open(SDL_joylist[joystick->index], O_RDONLY, 0);
@@ -475,31 +508,8 @@ int SDL_SYS_JoystickOpen(SDL_Joystick *joystick)
 #ifdef USE_INPUT_EVENTS
 	if ( ! EV_ConfigJoystick(joystick, fd) )
 #endif
-	{
-		if ( ioctl(fd, JSIOCGAXES, &n) < 0 ) {
-			joystick->naxes = 2;
-		} else {
-			joystick->naxes = n;
-		}
-		if ( ioctl(fd, JSIOCGBUTTONS, &n) < 0 ) {
-			joystick->nbuttons = 2;
-		} else {
-			joystick->nbuttons = n;
-		}
-#ifdef FANCY_HATS_AND_BALLS
-		/* Check for special joystick support */
-		name = SDL_SYS_JoystickName(joystick->index);
-		for ( i=0; special_joysticks[i]; ++i ) {
-			if (ConfigJoystick(joystick,name,special_joysticks[i])){
-				break;
-			}
-		}
-		if ( special_joysticks[i] == NULL ) {
-			ConfigJoystick(joystick, name,
-					getenv("SDL_LINUX_JOYSTICK"));
-		}
-#endif /* FANCY_HATS_AND_BALLS */
-	}
+		JS_ConfigJoystick(joystick, fd);
+
 	return(0);
 }
 
@@ -530,20 +540,6 @@ void HandleHat(SDL_Joystick *stick, Uint8 hat, int axis, int value)
 	}
 }
 
-/* This was necessary for the Wingman Extreme Analog joystick */
-static __inline__
-void HandleAnalogHat(SDL_Joystick *stick, Uint8 hat, int value)
-{
-	const Uint8 position_map[] = {
-		SDL_HAT_UP,
-		SDL_HAT_RIGHT,
-		SDL_HAT_DOWN,
-		SDL_HAT_LEFT,
-		SDL_HAT_CENTERED
-	};
-	SDL_PrivateJoystickHat(stick, hat, position_map[(value/16000)+2]);
-}
-
 static __inline__
 void HandleBall(SDL_Joystick *stick, Uint8 ball, int axis, int value)
 {
@@ -572,21 +568,12 @@ static __inline__ void JS_HandleEvents(SDL_Joystick *joystick)
 					break;
 				}
 				events[i].number -= joystick->naxes;
-				if ( joystick->hwdata->analog_hat ) {
-					other_axis = events[i].number;
-					if ( other_axis < joystick->nhats ) {
-						HandleAnalogHat(joystick, other_axis,
-							events[i].value);
-						break;
-					}
-				} else {
-					other_axis = (events[i].number / 2);
-					if ( other_axis < joystick->nhats ) {
-						HandleHat(joystick, other_axis,
-							events[i].number%2,
-							events[i].value);
-						break;
-					}
+				other_axis = (events[i].number / 2);
+				if ( other_axis < joystick->nhats ) {
+					HandleHat(joystick, other_axis,
+						events[i].number%2,
+						events[i].value);
+					break;
 				}
 				events[i].number -= joystick->nhats*2;
 				other_axis = (events[i].number / 2);
@@ -626,13 +613,11 @@ static __inline__ int EV_AxisCorrect(SDL_Joystick *joystick, int which, int valu
 		value *= correct->coef[2];
 		value >>= 14;
 	}
+
 	/* Clamp and return */
-	if ( value < -32767 ) {
-		value = -32767;
-	} else
-	if ( value > 32767 ) {
-		value = 32767;
-	}
+	if ( value < -32767 ) return -32767;
+	if ( value >  32767 ) return  32767;
+
 	return value;
 }
 
