@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysjoystick.c,v 1.3 2002/10/05 16:27:29 slouken Exp $";
+ "@(#) $Id: SDL_sysjoystick.c,v 1.4 2002/12/07 06:54:47 slouken Exp $";
 #endif
 
 /*
@@ -35,7 +35,7 @@ static char rcsid =
 #include <stdlib.h>
 #include <string.h>
 
-#include <sys/cookie.h>
+#include <mint/cookie.h>
 #include <mint/osbind.h>
 
 #include "SDL_types.h"
