@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_BeApp.cc,v 1.2 2001/04/26 16:50:17 hercules Exp $";
+ "@(#) $Id: SDL_BeApp.cc,v 1.5 2003/01/20 02:34:04 slouken Exp $";
 #endif
 
 /* Handle the BeApp specific portions of the application */
@@ -31,6 +31,7 @@ static char rcsid =
 #include <storage/Path.h>
 #include <storage/Entry.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include "SDL_BeApp.h"
