@@ -2,6 +2,11 @@
 #
 # Program to take a set of header files and generate DLL export definitions
 
+# Export Objective-C classes for Cocoa integration support
+print "\t.objc_class_name_SDL_QuartzWindow\n";
+print "\t.objc_class_name_SDL_QuartzWindowView\n";
+print "\t.objc_class_name_SDL_QuartzWindowDelegate\n";
+
 while ( ($file = shift(@ARGV)) ) {
 	if ( ! defined(open(FILE, $file)) ) {
 		warn "Couldn't open $file: $!\n";
@@ -10,9 +15,7 @@ while ( ($file = shift(@ARGV)) ) {
 	$printed_header = 0;
 	$file =~ s,.*/,,;
 	while (<FILE>) {
-		if ( /DECLSPEC.*\s\**([^\s\(]+)\(/ ) {
-			print "\t_$1\n";
-		} elsif ( /DECLSPEC.*\s\**([^\s\(]+)$/ ) {
+		if ( / DECLSPEC.*SDLCALL ([^\s\(]+)/ ) {
 			print "\t_$1\n";
 		}
 	}
