@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_systhread.c,v 1.5 2002/04/01 15:35:28 slouken Exp $";
+ "@(#) $Id: SDL_systhread.c,v 1.6 2002/06/10 20:42:02 slouken Exp $";
 #endif
 
 /*
@@ -64,7 +64,8 @@ int SDL_SYS_CreateThread(SDL_Thread *thread, void *args)
 	pth_attr_set(type, PTH_ATTR_JOINABLE, TRUE);
 
 	/* Create the thread and go! */
-	if ( pth_spawn(type, RunThread, args) == NULL ) {
+	thread->handle = pth_spawn(type, RunThread, args);
+	if ( thread->handle == NULL ) {
 		SDL_SetError("Not enough resources to create thread");
 		return(-1);
 	}
