@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_gamma.c,v 1.3 2001/08/18 21:58:26 slouken Exp $";
+ "@(#) $Id: SDL_gamma.c,v 1.6 2003/08/08 15:08:41 slouken Exp $";
 #endif
 
 /* Gamma correction support */
@@ -151,7 +151,7 @@ int SDL_GetGamma(float *red, float *green, float *blue)
 	return succeeded;
 }
 
-int SDL_SetGammaRamp(Uint16 *red, Uint16 *green, Uint16 *blue)
+int SDL_SetGammaRamp(const Uint16 *red, const Uint16 *green, const Uint16 *blue)
 {
 	int succeeded;
 	SDL_VideoDevice *video = current_video;
