@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_stretch.c,v 1.6 2002/03/06 11:23:03 slouken Exp $";
+ "@(#) $Id: SDL_stretch.c,v 1.7 2003/05/29 04:25:29 slouken Exp $";
 #endif
 
 /* This a stretch blit implementation based on ideas given to me by
@@ -261,9 +261,8 @@ int SDL_SoftStretch(SDL_Surface *src, SDL_Rect *srcrect,
 			break;
 		    default:
 #ifdef __GNUC__
-			__asm__ __volatile__ ("
-				call _copy_row
-			"
+			__asm__ __volatile__ (
+			"call _copy_row"
 			: "=&D" (u1), "=&S" (u2)
 			: "0" (dstp), "1" (srcp)
 			: "memory" );
