@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_surface.c,v 1.3 2001/07/31 05:36:10 slouken Exp $";
+ "@(#) $Id: SDL_surface.c,v 1.15 2002/10/15 05:22:50 slouken Exp $";
 #endif
 
 #include <stdio.h>
@@ -107,6 +107,7 @@ SDL_Surface * SDL_CreateRGBSurface (Uint32 flags,
 	surface->locked = 0;
 	surface->map = NULL;
 	surface->format_version = 0;
+	surface->unused1 = 0;
 	SDL_SetClipRect(surface, NULL);
 
 	/* Get the pixels */
@@ -211,6 +212,7 @@ int SDL_SetColorKey (SDL_Surface *surface, Uint32 flag, Uint32 key)
 	SDL_InvalidateMap(surface->map);
 	return(0);
 }
+/* This function sets the alpha channel of a surface */
 int SDL_SetAlpha (SDL_Surface *surface, Uint32 flag, Uint8 value)
 {
 	Uint32 oldflags = surface->flags;
@@ -269,6 +271,52 @@ int SDL_SetAlpha (SDL_Surface *surface, Uint32 flag, Uint8 value)
 		SDL_InvalidateMap(surface->map);
 	return(0);
 }
+int SDL_SetAlphaChannel(SDL_Surface *surface, Uint8 value)
+{
+	int row, col;
+	int offset;
+	Uint8 *buf;
+
+	if ( (surface->format->Amask != 0xFF000000) &&
+	     (surface->format->Amask != 0x000000FF) ) {
+		SDL_SetError("Unsupported surface alpha mask format");
+		return -1;
+	}
+
+#if SDL_BYTEORDER == SDL_LIL_ENDIAN
+	if ( surface->format->Amask == 0xFF000000 ) {
+			offset = 3;
+	} else {
+			offset = 0;
+	}
+#else
+	if ( surface->format->Amask == 0xFF000000 ) {
+			offset = 0;
+	} else {
+			offset = 3;
+	}
+#endif /* Byte ordering */
+
+	/* Quickly set the alpha channel of an RGBA or ARGB surface */
+	if ( SDL_MUSTLOCK(surface) ) {
+		if ( SDL_LockSurface(surface) < 0 ) {
+			return -1;
+		}
+	}
+	row = surface->h;
+	while (row--) {
+		col = surface->w;
+		buf = (Uint8 *)surface->pixels + row * surface->pitch + offset;
+		while(col--) {
+			*buf = value;
+			buf += 4;
+		}
+	}
+	if ( SDL_MUSTLOCK(surface) ) {
+		SDL_UnlockSurface(surface);
+	}
+	return 0;
+}
 
 /*
  * A function to calculate the intersection of two rectangles:
@@ -351,6 +399,8 @@ int SDL_LowerBlit (SDL_Surface *src, SDL_Rect *srcrect,
 				SDL_Surface *dst, SDL_Rect *dstrect)
 {
 	SDL_blit do_blit;
+	SDL_Rect hw_srcrect;
+	SDL_Rect hw_dstrect;
 
 	/* Check to make sure the blit mapping is valid */
 	if ( (src->map->dst != dst) ||
@@ -362,6 +412,18 @@ int SDL_LowerBlit (SDL_Surface *src, SDL_Rect *srcrect,
 
 	/* Figure out which blitter to use */
 	if ( (src->flags & SDL_HWACCEL) == SDL_HWACCEL ) {
+		if ( src == SDL_VideoSurface ) {
+			hw_srcrect = *srcrect;
+			hw_srcrect.x += current_video->offset_x;
+			hw_srcrect.y += current_video->offset_y;
+			srcrect = &hw_srcrect;
+		}
+		if ( dst == SDL_VideoSurface ) {
+			hw_dstrect = *dstrect;
+			hw_dstrect.x += current_video->offset_x;
+			hw_dstrect.y += current_video->offset_y;
+			dstrect = &hw_dstrect;
+		}
 		do_blit = src->map->hw_blit;
 	} else {
 		do_blit = src->map->sw_blit;
@@ -462,6 +524,20 @@ int SDL_UpperBlit (SDL_Surface *src, SDL_Rect *srcrect,
 	return 0;
 }
 
+static int SDL_FillRect1(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
+{
+	/* FIXME: We have to worry about packing order.. *sigh* */
+	SDL_SetError("1-bpp rect fill not yet implemented");
+	return -1;
+}
+
+static int SDL_FillRect4(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
+{
+	/* FIXME: We have to worry about packing order.. *sigh* */
+	SDL_SetError("4-bpp rect fill not yet implemented");
+	return -1;
+}
+
 /* 
  * This function performs a fast fill of the given rectangle with 'color'
  */
@@ -472,6 +548,22 @@ int SDL_FillRect(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
 	int x, y;
 	Uint8 *row;
 
+	/* This function doesn't work on surfaces < 8 bpp */
+	if ( dst->format->BitsPerPixel < 8 ) {
+		switch(dst->format->BitsPerPixel) {
+		    case 1:
+			return SDL_FillRect1(dst, dstrect, color);
+			break;
+		    case 4:
+			return SDL_FillRect4(dst, dstrect, color);
+			break;
+		    default:
+			SDL_SetError("Fill rect on unsupported surface format");
+			return(-1);
+			break;
+		}
+	}
+
 	/* If 'dstrect' == NULL, then fill the whole surface */
 	if ( dstrect ) {
 		/* Perform clipping */
@@ -485,6 +577,13 @@ int SDL_FillRect(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
 	/* Check for hardware acceleration */
 	if ( ((dst->flags & SDL_HWSURFACE) == SDL_HWSURFACE) &&
 					video->info.blit_fill ) {
+		SDL_Rect hw_rect;
+		if ( dst == SDL_VideoSurface ) {
+			hw_rect = *dstrect;
+			hw_rect.x += current_video->offset_x;
+			hw_rect.y += current_video->offset_y;
+			dstrect = &hw_rect;
+		}
 		return(video->FillHWRect(this, dst, dstrect, color));
 	}
 
@@ -623,10 +722,6 @@ int SDL_FillRect(SDL_Surface *dst, SDL_Rect *dstrect, Uint32 color)
 
 /*
  * Lock a surface to directly access the pixels
- * -- Do not call this from any blit function, as SDL_DrawCursor() may recurse
- *    Instead, use:
- *    if ( (surface->flags & SDL_HWSURFACE) == SDL_HWSURFACE )
- *               video->LockHWSurface(video, surface);
  */
 int SDL_LockSurface (SDL_Surface *surface)
 {
@@ -655,10 +750,6 @@ int SDL_LockSurface (SDL_Surface *surface)
 }
 /*
  * Unlock a previously locked surface
- * -- Do not call this from any blit function, as SDL_DrawCursor() may recurse
- *    Instead, use:
- *    if ( (surface->flags & SDL_HWSURFACE) == SDL_HWSURFACE )
- *               video->UnlockHWSurface(video, surface);
  */
 void SDL_UnlockSurface (SDL_Surface *surface)
 {
@@ -711,6 +802,14 @@ SDL_Surface * SDL_ConvertSurface (SDL_Surface *surface,
 		}
 	}
 
+	/* Only create hw surfaces with alpha channel if hw alpha blits
+	   are supported */
+	if(format->Amask != 0 && (flags & SDL_HWSURFACE)) {
+		const SDL_VideoInfo *vi = SDL_GetVideoInfo();
+		if(!vi || !vi->blit_hw_A)
+			flags &= ~SDL_HWSURFACE;
+	}
+
 	/* Create a new surface with the desired format */
 	convert = SDL_CreateRGBSurface(flags,
 				surface->w, surface->h, format->BitsPerPixel,
@@ -740,8 +839,13 @@ SDL_Surface * SDL_ConvertSurface (SDL_Surface *surface,
 		}
 	}
 	if ( (surface_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
-		alpha = surface->format->alpha;
-		SDL_SetAlpha(surface, 0, 0);
+		/* Copy over the alpha channel to RGBA if requested */
+		if ( format->Amask ) {
+			surface->flags &= ~SDL_SRCALPHA;
+		} else {
+			alpha = surface->format->alpha;
+			SDL_SetAlpha(surface, 0, 0);
+		}
 	}
 
 	/* Copy over the image data */
@@ -772,7 +876,11 @@ SDL_Surface * SDL_ConvertSurface (SDL_Surface *surface,
 		        SDL_SetAlpha(convert, aflags|(flags&SDL_RLEACCELOK),
 				alpha);
 		}
-		SDL_SetAlpha(surface, aflags, alpha);
+		if ( format->Amask ) {
+			surface->flags |= SDL_SRCALPHA;
+		} else {
+			SDL_SetAlpha(surface, aflags, alpha);
+		}
 	}
 
 	/* We're ready to go! */
@@ -804,7 +912,7 @@ void SDL_FreeSurface (SDL_Surface *surface)
 		SDL_FreeBlitMap(surface->map);
 		surface->map = NULL;
 	}
-	if ( (surface->flags & SDL_HWSURFACE) == SDL_HWSURFACE ) {
+	if ( surface->hwdata ) {
 		SDL_VideoDevice *video = current_video;
 		SDL_VideoDevice *this  = current_video;
 		video->FreeHWSurface(this, surface);
