@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_sysvideo.h,v 1.19 2002/10/05 16:50:56 slouken Exp $";
+ "@(#) $Id: SDL_sysvideo.h,v 1.22 2003/07/22 15:33:28 slouken Exp $";
 #endif
 
 #ifndef _SDL_sysvideo_h
@@ -304,6 +304,8 @@ struct SDL_VideoDevice {
 		int accum_blue_size;
 		int accum_alpha_size;
 		int stereo;
+		int multisamplebuffers;
+		int multisamplesamples;
 		int driver_loaded;
 		char driver_path[256];
 		void* dll_handle;
@@ -409,6 +411,9 @@ extern VideoBootStrap PG_bootstrap;
 #ifdef ENABLE_DC
 extern VideoBootStrap DC_bootstrap;
 #endif
+#ifdef ENABLE_RISCOS
+extern VideoBootStrap RISCOS_bootstrap;
+#endif
 /* This is the current video device */
 extern SDL_VideoDevice *current_video;
 
