@@ -1,6 +1,6 @@
 /*
 	SDL - Simple DirectMedia Layer
-	Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+	Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 	Sam Lantinga
-	slouken@devolution.com
+	slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_fbmatrox.c,v 1.5 2001/07/14 19:10:06 hercules Exp $";
+ "@(#) $Id: SDL_fbmatrox.c,v 1.7 2002/03/06 11:23:03 slouken Exp $";
 #endif
 
 #include "SDL_types.h"
