@@ -1,6 +1,6 @@
 /*
 	SDL - Simple DirectMedia Layer
-	Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+	Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 	Sam Lantinga
-	slouken@devolution.com
+	slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_fbriva.h,v 1.2 2001/08/01 03:23:40 slouken Exp $";
+ "@(#) $Id: SDL_fbriva.h,v 1.4 2002/03/06 11:23:04 slouken Exp $";
 #endif
 
 /* Riva hardware acceleration for the SDL framebuffer console driver */
