@@ -25,7 +25,7 @@
 */
 
 #include "SDL_nxvideo.h"
-#include <SDL/SDL.h>
+#include <SDL.h>
 
 extern SDL_Rect ** NX_ListModes (_THIS, SDL_PixelFormat * format, Uint32 flags) ;
 extern void NX_FreeVideoModes (_THIS) ;
