@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,18 +17,23 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
+#ifndef __SDL_PH_EVENTS_H__
+#define __SDL_PH_EVENTS_H__
+
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ph_events_c.h,v 1.2 2001/04/26 16:50:19 hercules Exp $";
-#endif
+ "@(#) $Id: SDL_ph_events_c.h,v 1.6 2003/08/30 17:07:59 slouken Exp $";
+#endif /* SAVE_RCSID */
 
 #include "SDL_ph_video.h"
 
-#define EVENT_SIZE sizeof( PhEvent_t ) + 1000
+#define EVENT_SIZE sizeof(PhEvent_t) + 1000
 
 /* Functions to be exported */
 extern void ph_InitOSKeymap(_THIS);
 extern void ph_PumpEvents(_THIS);
+
+#endif /* __SDL_PH_EVENTS_H__ */
