@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,13 +17,16 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
+#ifndef __SDL_PH_MOUSE_H__
+#define __SDL_PH_MOUSE_H__
+
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_ph_mouse_c.h,v 1.2 2001/04/26 16:50:19 hercules Exp $";
-#endif
+ "@(#) $Id: SDL_ph_mouse_c.h,v 1.5 2003/08/30 17:07:59 slouken Exp $";
+#endif /* SAVE_RCSID */
 
 #include "SDL_ph_video.h"
 
@@ -35,3 +38,6 @@ extern PhCursorDef_t ph_GetWMPhCursor(WMcursor *cursor);
 extern int ph_ShowWMCursor(_THIS, WMcursor *cursor);
 extern void ph_WarpWMCursor(_THIS, Uint16 x, Uint16 y);
 extern void ph_CheckMouseMode(_THIS);
+extern void ph_UpdateMouse(_THIS);
+
+#endif /* __SDL_PH_MOUSE_H__ */
