@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_lowvideo.h,v 1.4 2001/08/09 12:21:32 slouken Exp $";
+ "@(#) $Id: SDL_lowvideo.h,v 1.12 2002/08/20 04:55:36 slouken Exp $";
 #endif
 
 #ifndef _SDL_lowvideo_h
@@ -35,21 +35,29 @@ static char rcsid =
 /* Hidden "this" pointer for the video functions */
 #define _THIS	SDL_VideoDevice *this
 
+#define WINDIB_FULLSCREEN()						\
+(									\
+	SDL_VideoSurface &&						\
+	((SDL_VideoSurface->flags & SDL_FULLSCREEN) == SDL_FULLSCREEN) && \
+	(((SDL_VideoSurface->flags & SDL_OPENGL   ) == SDL_OPENGL    ) || \
+	 (strcmp(this->name, "windib") == 0))				\
+)
 #define DDRAW_FULLSCREEN() 						\
 (									\
+	SDL_VideoSurface &&						\
 	((SDL_VideoSurface->flags & SDL_FULLSCREEN) == SDL_FULLSCREEN) && \
 	((SDL_VideoSurface->flags & SDL_OPENGL    ) != SDL_OPENGL    ) && \
 	(strcmp(this->name, "directx") == 0)				\
 )
 
-#define DINPUT_FULLSCREEN() 						\
-(									\
-	((SDL_VideoSurface->flags & SDL_FULLSCREEN) == SDL_FULLSCREEN) && \
-	(strcmp(this->name, "directx") == 0)				\
-)
+#define DINPUT_FULLSCREEN()	DDRAW_FULLSCREEN()
 
 /* The main window -- and a function to set it for the audio */
-extern const char *SDL_Appname;
+#ifdef _WIN32_WCE
+extern LPWSTR SDL_Appname;
+#else
+extern LPSTR SDL_Appname;
+#endif
 extern HINSTANCE SDL_Instance;
 extern HWND SDL_Window;
 extern const char *SDL_windowid;
@@ -63,9 +71,6 @@ extern void (*WIN_RealizePalette)(_THIS);
 /* Called by windows message loop when the system palette changes */
 extern void (*WIN_PaletteChanged)(_THIS, HWND window);
 
-/* Called by windows message loop when losing or gaining gamma focus */
-extern void (*WIN_SwapGamma)(_THIS);
-
 /* Called by windows message loop when a portion of the screen needs update */
 extern void (*WIN_WinPAINT)(_THIS, HDC hdc);
 
@@ -84,6 +89,14 @@ extern int SDL_resizing;
 /* Flag -- the mouse is in relative motion mode */
 extern int mouse_relative;
 
+/* The GDI fullscreen mode currently active */
+#ifndef NO_CHANGEDISPLAYSETTINGS
+extern DEVMODE SDL_fullscreen_mode;
+#endif
+
+/* The system gamma ramp for GDI modes */
+extern WORD *gamma_saved;
+
 /* This is really from SDL_dx5audio.c */
 extern void DX5_SoundFocus(HWND window);
 
