@@ -1,6 +1,6 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dx5events.c,v 1.4 2001/08/09 12:21:32 slouken Exp $";
+ "@(#) $Id: SDL_dx5events.c,v 1.17 2002/11/09 05:47:24 slouken Exp $";
 #endif
 
 /* CAUTION!!!!  If you modify this file, check ../windib/SDL_sysevents.c */
@@ -43,9 +43,13 @@ static char rcsid =
 #define WM_APP	0x8000
 #endif
 
+#ifdef _WIN32_WCE
+#define NO_GETKEYBOARDSTATE
+#endif
+
 /* The keyboard and mouse device input */
 #define MAX_INPUTS	16		/* Maximum of 16-1 input devices */
-#define INPUT_QSIZE	32		/* Buffer up to 32 input messages */
+#define INPUT_QSIZE	512		/* Buffer up to 512 input messages */
 
 static LPDIRECTINPUT dinput = NULL;
 static LPDIRECTINPUTDEVICE2 SDL_DIdev[MAX_INPUTS];
@@ -54,6 +58,7 @@ static void (*SDL_DIfun[MAX_INPUTS])(const int, DIDEVICEOBJECTDATA *);
 static int SDL_DIndev = 0;
 static int mouse_lost;
 static int mouse_pressed;
+static int mouse_buttons_swapped = 0;
 
 /* The translation table from a DirectInput scancode to an SDL keysym */
 static SDLKey DIK_keymap[256];
@@ -67,7 +72,7 @@ static WNDPROC userWindowProc = NULL;
 static void SetDIerror(char *function, int code)
 {
 	static char *error;
-	static char  errbuf[BUFSIZ];
+	static char  errbuf[1024];
 
 	errbuf[0] = 0;
 	switch (code) {
@@ -212,38 +217,12 @@ static int DX5_DInputInit(_THIS)
 		++SDL_DIndev;
 	}
 	mouse_pressed = 0;
+	mouse_buttons_swapped = GetSystemMetrics(SM_SWAPBUTTON);
 
 	/* DirectInput is ready! */
 	return(0);
 }
 
-/* Change cooperative level based on whether or not we are fullscreen */
-void DX5_DInputReset(_THIS, int fullscreen)
-{
-	DWORD level;
-	int i;
-	HRESULT result;
-
-	for ( i=0; i<MAX_INPUTS; ++i ) {
-		if ( SDL_DIdev[i] != NULL ) {
-			if ( fullscreen ) {
-				level = inputs[i].raw_level;
-			} else {
-				level = inputs[i].win_level;
-			}
-			IDirectInputDevice2_Unacquire(SDL_DIdev[i]);
-			result = IDirectInputDevice2_SetCooperativeLevel(
-					SDL_DIdev[i], SDL_Window, level);
-			IDirectInputDevice2_Acquire(SDL_DIdev[i]);
-			if ( result != DI_OK ) {
-				SetDIerror(
-			"DirectInputDevice::SetCooperativeLevel", result);
-			}
-		}
-	}
-	mouse_lost = 1;
-}
-
 /* Clean up DirectInput */
 static void DX5_DInputQuit(_THIS)
 {
@@ -296,9 +275,11 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 	Sint16 xrel, yrel;
 	Uint8 state;
 	Uint8 button;
+	DWORD timestamp = 0;
 
 	/* If we are in windowed mode, Windows is taking care of the mouse */
-	if ( ! (SDL_PublicSurface->flags & SDL_FULLSCREEN) ) {
+	if (  (SDL_PublicSurface->flags & SDL_OPENGL) ||
+	     !(SDL_PublicSurface->flags & SDL_FULLSCREEN) ) {
 		return;
 	}
 
@@ -362,6 +343,11 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 					}
 					state = SDL_RELEASED;
 				}
+				if ( mouse_buttons_swapped ) {
+					if ( button == 1 ) button = 3;
+					else
+					if ( button == 3 ) button = 1;
+				}
 				posted = SDL_PrivateMouseButton(state, button,
 									0, 0);
 			}
@@ -378,9 +364,27 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 	for ( i=0; i<(int)numevents; ++i ) {
 		switch (ptrbuf[i].dwOfs) {
 			case DIMOFS_X:
+				if ( timestamp != ptrbuf[i].dwTimeStamp ) {
+					if ( xrel || yrel ) {
+						posted = SDL_PrivateMouseMotion(
+								0, 1, xrel, yrel);
+						xrel = 0;
+						yrel = 0;
+					}
+					timestamp = ptrbuf[i].dwTimeStamp;
+				}
 				xrel += (Sint16)ptrbuf[i].dwData;
 				break;
 			case DIMOFS_Y:
+				if ( timestamp != ptrbuf[i].dwTimeStamp ) {
+					if ( xrel || yrel ) {
+						posted = SDL_PrivateMouseMotion(
+								0, 1, xrel, yrel);
+						xrel = 0;
+						yrel = 0;
+					}
+					timestamp = ptrbuf[i].dwTimeStamp;
+				}
 				yrel += (Sint16)ptrbuf[i].dwData;
 				break;
 			case DIMOFS_Z:
@@ -390,12 +394,15 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 					xrel = 0;
 					yrel = 0;
 				}
+				timestamp = 0;
 				if((int)ptrbuf[i].dwData > 0)
-					posted = SDL_PrivateMouseButton(
-							SDL_PRESSED, 4, 0, 0);
-				else if((int)ptrbuf[i].dwData < 0)
-					posted = SDL_PrivateMouseButton(
-							SDL_PRESSED, 5, 0, 0);
+					button = SDL_BUTTON_WHEELUP;
+				else
+					button = SDL_BUTTON_WHEELDOWN;
+				posted = SDL_PrivateMouseButton(
+						SDL_PRESSED, button, 0, 0);
+				posted |= SDL_PrivateMouseButton(
+						SDL_RELEASED, button, 0, 0);
 				break;
 			case DIMOFS_BUTTON0:
 			case DIMOFS_BUTTON1:
@@ -407,6 +414,7 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 					xrel = 0;
 					yrel = 0;
 				}
+				timestamp = 0;
 				button = (Uint8)(ptrbuf[i].dwOfs-DIMOFS_BUTTON0)+1;
 				/* Button #2 on two button mice is button 3
 				   (the middle button is button 2)
@@ -431,6 +439,11 @@ static void handle_mouse(const int numevents, DIDEVICEOBJECTDATA *ptrbuf)
 					}
 					state = SDL_RELEASED;
 				}
+				if ( mouse_buttons_swapped ) {
+					if ( button == 1 ) button = 3;
+					else
+					if ( button == 3 ) button = 1;
+				}
 				posted = SDL_PrivateMouseButton(state, button,
 									0, 0);
 				break;
@@ -446,6 +459,7 @@ LONG
  DX5_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
 	switch (msg) {
+#ifdef WM_ACTIVATEAPP
 		case WM_ACTIVATEAPP: {
 			int i, active;
 
@@ -464,7 +478,9 @@ LONG
 			}
 		}
 		break;
+#endif /* WM_ACTIVATEAPP */
 
+#ifdef WM_DISPLAYCHANGE
 		case WM_DISPLAYCHANGE: {
 			WORD BitsPerPixel;
 			WORD SizeX, SizeY;
@@ -476,6 +492,7 @@ LONG
 			/* We cause this message when we go fullscreen */
 		}
 		break;
+#endif /* WM_DISPLAYCHANGE */
 
 		/* The keyboard is handled via DirectInput */
 		case WM_SYSKEYUP:
@@ -486,6 +503,7 @@ LONG
 		}
 		return(0);
 
+#if defined(SC_SCREENSAVE) || defined(SC_MONITORPOWER)
 		/* Don't allow screen savers or monitor power downs.
 		   This is because they quietly clear DirectX surfaces.
 		   It would be better to allow the application to
@@ -498,11 +516,11 @@ LONG
 			    (wParam&0xFFF0)==SC_MONITORPOWER)
 				return(0);
 		}
-		goto custom_processing;
-		break;
+		/* Fall through to default processing */
+
+#endif /* SC_SCREENSAVE || SC_MONITORPOWER */
 
 		default: {
-		custom_processing:
 			/* Only post the event if we're watching for it */
 			if ( SDL_ProcessEvents[SDL_SYSWMEVENT] == SDL_ENABLE ) {
 			        SDL_SysWMmsg wmmsg;
@@ -531,7 +549,7 @@ LONG
    1 if there was input, 0 if there was no input, or -1 if the application has
    posted a quit message.
 */
-static int DX5_CheckInput(_THIS, int timeout)
+static int DX5_CheckInput(_THIS, int timeout, BOOL processInput)
 {
 	MSG msg;
 	int      i;
@@ -541,8 +559,8 @@ static int DX5_CheckInput(_THIS, int timeout)
 	/* Check the normal windows queue (highest preference) */
 	posted = 0;
 	while ( ! posted &&
-	        PeekMessage(&msg, NULL, 0, (WM_APP-1), PM_NOREMOVE) ) {
-		if ( GetMessage(&msg, NULL, 0, (WM_APP-1)) > 0 ) {
+	        PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) ) {
+		if ( GetMessage(&msg, NULL, 0, 0) > 0 ) {
 			DispatchMessage(&msg);
 		} else {
 			return(-1);
@@ -590,7 +608,7 @@ static int DX5_CheckInput(_THIS, int timeout)
 							evtbuf, &numevents, 0);
 		}
 		/* Handle the events */
-		if ( result == DI_OK ) {
+		if ( result == DI_OK && processInput ) {
 			/* Note: This can post multiple events to event queue
 			 */
 			(*SDL_DIfun[event])((int)numevents, evtbuf);
@@ -611,16 +629,52 @@ static int DX5_CheckInput(_THIS, int timeout)
 	return(0);
 }
 
+/* Change cooperative level based on whether or not we are fullscreen */
+void DX5_DInputReset(_THIS, int fullscreen)
+{
+	DWORD level;
+	int i;
+	HRESULT result;
+
+	for ( i=0; i<MAX_INPUTS; ++i ) {
+		if ( SDL_DIdev[i] != NULL ) {
+			if ( fullscreen ) {
+				level = inputs[i].raw_level;
+			} else {
+				level = inputs[i].win_level;
+			}
+			IDirectInputDevice2_Unacquire(SDL_DIdev[i]);
+			result = IDirectInputDevice2_SetCooperativeLevel(
+					SDL_DIdev[i], SDL_Window, level);
+			IDirectInputDevice2_Acquire(SDL_DIdev[i]);
+			if ( result != DI_OK ) {
+				SetDIerror(
+			"DirectInputDevice::SetCooperativeLevel", result);
+			}
+		}
+	}
+	mouse_lost = 1;
+
+	/* Flush pending input */
+	DX5_CheckInput(this, 0, FALSE);
+}
+
 void DX5_PumpEvents(_THIS)
 {
 	/* Wait for messages and DirectInput */
-	while ( DX5_CheckInput(this, 0) > 0 ) {
+	while ( DX5_CheckInput(this, 0, TRUE) > 0 ) {
 		/* Loop and check again */;
 	}
 }
 
 void DX5_InitOSKeymap(_THIS)
 {
+#ifndef DIK_PAUSE
+#define DIK_PAUSE	0xC5
+#endif
+#ifndef DIK_OEM_102
+#define DIK_OEM_102	0x56	/* < > | on UK/Germany keyboards */
+#endif
 	int i;
 
 	/* Map the DIK scancodes to SDL keysyms */
@@ -671,6 +725,7 @@ void DX5_InitOSKeymap(_THIS)
 	DIK_keymap[DIK_GRAVE] = SDLK_BACKQUOTE;
 	DIK_keymap[DIK_LSHIFT] = SDLK_LSHIFT;
 	DIK_keymap[DIK_BACKSLASH] = SDLK_BACKSLASH;
+	DIK_keymap[DIK_OEM_102] = SDLK_BACKSLASH;
 	DIK_keymap[DIK_Z] = SDLK_z;
 	DIK_keymap[DIK_X] = SDLK_x;
 	DIK_keymap[DIK_C] = SDLK_c;
@@ -724,6 +779,7 @@ void DX5_InitOSKeymap(_THIS)
 	DIK_keymap[DIK_DIVIDE] = SDLK_KP_DIVIDE;
 	DIK_keymap[DIK_SYSRQ] = SDLK_SYSREQ;
 	DIK_keymap[DIK_RMENU] = SDLK_RALT;
+	DIK_keymap[DIK_PAUSE] = SDLK_PAUSE;
 	DIK_keymap[DIK_HOME] = SDLK_HOME;
 	DIK_keymap[DIK_UP] = SDLK_UP;
 	DIK_keymap[DIK_PRIOR] = SDLK_PAGEUP;
@@ -748,14 +804,21 @@ static SDL_keysym *TranslateKey(UINT scancode, SDL_keysym *keysym, int pressed)
 	keysym->unicode = 0;
 	if ( pressed && SDL_TranslateUNICODE ) { /* Someday use ToUnicode() */
 		UINT vkey;
+#ifndef NO_GETKEYBOARDSTATE
 		BYTE keystate[256];
 		BYTE chars[2];
+#endif
 
 		vkey = MapVirtualKey(scancode, 1);
+#ifdef NO_GETKEYBOARDSTATE
+		/* Uh oh, better hope the vkey is close enough.. */
+		keysym->unicode = vkey;
+#else
 		GetKeyboardState(keystate);
 		if ( ToAscii(vkey,scancode,keystate,(WORD *)chars,0) == 1 ) {
 			keysym->unicode = chars[0];
 		}
+#endif
 	}
 	return(keysym);
 }
@@ -771,18 +834,20 @@ int DX5_CreateWindow(_THIS)
 		SDL_DIfun[i] = NULL;
 	}
 
-	/* Create the SDL window */
+#ifndef CS_BYTEALIGNCLIENT
+#define CS_BYTEALIGNCLIENT	0
+#endif
 	SDL_RegisterApp("SDL_app", CS_BYTEALIGNCLIENT, 0);
 	if ( SDL_windowid ) {
 		SDL_Window = (HWND)strtol(SDL_windowid, NULL, 0);
 
-      /* DJM: we want all event's for the user specified
-         window to be handled by SDL.
-       */
-      if (SDL_Window) {
-         userWindowProc = (WNDPROC)GetWindowLong(SDL_Window, GWL_WNDPROC);
-         SetWindowLong(SDL_Window, GWL_WNDPROC, (LONG)WinMessage);
-      }
+		/* DJM: we want all event's for the user specified
+		   window to be handled by SDL.
+		 */
+		if (SDL_Window) {
+			userWindowProc = (WNDPROC)GetWindowLong(SDL_Window, GWL_WNDPROC);
+			SetWindowLong(SDL_Window, GWL_WNDPROC, (LONG)WinMessage);
+		}
 	} else {
 		SDL_Window = CreateWindow(SDL_Appname, SDL_Appname,
                         (WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX),
