@@ -15,7 +15,7 @@ SDL_Surface *LoadIconSurface(char *file, Uint8 **maskp)
 	SDL_Surface *icon;
 	Uint8       *pixels;
 	Uint8       *mask;
-	int          mlen, i;
+	int          mlen, i, j;
 
 	*maskp = NULL;
 
@@ -26,12 +26,15 @@ SDL_Surface *LoadIconSurface(char *file, Uint8 **maskp)
 		return(NULL);
 	}
 
-	/* Check width and height */
+	/* Check width and height 
 	if ( (icon->w%8) != 0 ) {
 		fprintf(stderr, "Icon width must be a multiple of 8!\n");
 		SDL_FreeSurface(icon);
 		return(NULL);
 	}
+*/
+    
+    
 	if ( icon->format->palette == NULL ) {
 		fprintf(stderr, "Icon must have a palette!\n");
 		SDL_FreeSurface(icon);
@@ -47,21 +50,21 @@ SDL_Surface *LoadIconSurface(char *file, Uint8 **maskp)
 				icon->format->palette->colors[*pixels].r,
 				icon->format->palette->colors[*pixels].g,
 				icon->format->palette->colors[*pixels].b);
-	mlen = icon->w*icon->h;
-	mask = (Uint8 *)malloc(mlen/8);
+	mlen = (icon->w*icon->h + 7) / 8;
+	mask = (Uint8 *)malloc(mlen);
 	if ( mask == NULL ) {
 		fprintf(stderr, "Out of memory!\n");
 		SDL_FreeSurface(icon);
 		return(NULL);
 	}
-	memset(mask, 0, mlen/8);
-	for ( i=0; i<mlen; ) {
-		if ( pixels[i] != *pixels )
-			mask[i/8] |= 0x01;
-		++i;
-		if ( (i%8) != 0 )
-			mask[i/8] <<= 1;
-	}
+	memset(mask, 0, mlen);
+	for ( i=0; i < icon->h; i++)
+        for (j=0; j < icon->w; j++) {
+            int pindex = i * icon->pitch + j;
+            int mindex = i * icon->w + j;
+            if ( pixels[pindex] != *pixels )
+                mask[mindex>>3] |= 1 << (7 - (mindex & 7));
+        }
 	*maskp = mask;
 	return(icon);
 }
@@ -90,7 +93,7 @@ void HotKey_ToggleGrab(void)
 	} else {
 		printf("Grab was off\n");
 	}
-	mode = SDL_WM_GrabInput(!mode);
+	mode = SDL_WM_GrabInput(mode ? SDL_GRAB_OFF : SDL_GRAB_ON);
 	if ( mode == SDL_GRAB_ON ) {
 		printf("Grab is now on\n");
 	} else {
@@ -254,6 +257,7 @@ int main(int argc, char *argv[])
 	SDL_Surface *icon;
 	Uint8 *icon_mask;
 	int parsed;
+	int w, h;
 
 	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
 		fprintf(stderr,
@@ -263,6 +267,8 @@ int main(int argc, char *argv[])
 	atexit(SDL_Quit);
 
 	/* Check command line arguments */
+	w = 640;
+	h = 480;
 	video_bpp = 8;
 	video_flags = SDL_SWSURFACE;
 	parsed = 1;
@@ -282,6 +288,16 @@ int main(int argc, char *argv[])
 			argc -= 1;
 			argv += 1;
 		} else
+		if ( (argc >= 3) && (strcmp(argv[1], "-width") == 0) ) {
+			w = atoi(argv[2]);
+			argc -= 2;
+			argv += 2;
+		} else
+		if ( (argc >= 3) && (strcmp(argv[1], "-height") == 0) ) {
+			h = atoi(argv[2]);
+			argc -= 2;
+			argv += 2;
+		} else
 		if ( (argc >= 3) && (strcmp(argv[1], "-bpp") == 0) ) {
 			video_bpp = atoi(argv[2]);
 			argc -= 2;
@@ -314,7 +330,7 @@ int main(int argc, char *argv[])
 		printf("No window title was set!\n");
 
 	/* Initialize the display */
-	if ( SetVideoMode(640, 480) < 0 ) {
+	if ( SetVideoMode(w, h) < 0 ) {
 		return(1);
 	}
 
